Package: staat1cho
Title: Study Indicators Based on Dutch Higher Education Data (1CHO)
Version: 0.1.0
Authors@R: c(
    person("Aslam", "Tanjung", email = "aslam.tanjung@surf.nl", role = c("aut", "cre")),
    person("Veerle", "van Son", role = "aut"),
    person("Damiëtte", "Bakx-van den Brink", role = "aut")
  )
Description: Calculates enrolment, graduation, dropout, and programme-switch
    indicators from the Dutch higher education registration data (1CHO)
    supplied by DUO. Includes an interactive 'Shiny' dashboard for exploring
    results.
License: MIT + file LICENSE
URL: https://github.com/cedanl/staat-van-onderwijsinstelling
BugReports: https://github.com/cedanl/staat-van-onderwijsinstelling/issues
Encoding: UTF-8
Language: nl
RoxygenNote: 7.3.3
Depends: R (>= 4.1.0)
Imports: cli, dplyr, forcats, readr, rlang, shiny
Suggests: bslib, DT, ggplot2, knitr, plotly, rmarkdown, scales,
        testthat (>= 3.0.0), tibble, tidyr
VignetteBuilder: knitr
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-05-13 12:27:33 UTC; Aslam
Author: Aslam Tanjung [aut, cre],
  Veerle van Son [aut],
  Damiëtte Bakx-van den Brink [aut]
Maintainer: Aslam Tanjung <aslam.tanjung@surf.nl>
Repository: CRAN
Date/Publication: 2026-05-18 18:40:02 UTC
Built: R 4.5.2; ; 2026-05-18 19:42:31 UTC; unix

# Generated by roxygen2: do not edit by hand

export(bereken_rendement)
export(bereken_studiewissel)
export(bereken_uitval)
export(combineer_indicatoren)
export(maak_basisbestand)
export(maak_diploma_behaald)
export(maak_instroom_cohort)
export(start_dashboard)
importFrom(rlang,":=")
importFrom(rlang,.data)

library(shiny)
library(bslib)

options(shiny.maxRequestSize = 500 * 1024^2) ## 500 MB
library(dplyr)
library(ggplot2)
library(forcats)
library(tidyr)
library(scales)
library(plotly)
library(DT)
library(readr)

library(staat1cho)

## Fonts ----

if (
  requireNamespace("showtext", quietly = TRUE) &&
    requireNamespace("sysfonts", quietly = TRUE)
) {
  sysfonts::font_add_google("Plus Jakarta Sans", "plus_jakarta")
  showtext::showtext_auto()
  FONT_FMLY <- "plus_jakarta"
} else {
  FONT_FMLY <- "sans"
}

## Npuls kleuren ----

NPULS_BLAUW <- "#3D68EC"
NPULS_ORANJE <- "#DD784B"
NPULS_GROEN <- "#00AF81"
NPULS_GEEL <- "#F4D74B"
NPULS_ROZE <- "#F4D9DC"
NPULS_ZWART <- "#000000"
NPULS_WIT <- "#FFFFFF"
NPULS_LICHT_GROEN <- "#CCEEE6"
KLEUR_NEUTRAAL <- "#6B7280"

KLEUREN_TERMIJN <- c(
  "3 jaar" = NPULS_ORANJE,
  "5 jaar" = NPULS_BLAUW,
  "8 jaar" = NPULS_GROEN
)

KLEUREN_UITVAL <- c(
  "binnen 1 jaar" = NPULS_ORANJE,
  "binnen 3 jaar" = NPULS_BLAUW
)

KLEUREN_WISSEL <- c(
  "binnen 1 jaar" = NPULS_GEEL,
  "binnen 3 jaar" = NPULS_GROEN
)

KLEUREN <- c(
  ## Status
  "Diploma behaald" = NPULS_GROEN,
  "Zittend" = NPULS_BLAUW,
  "Uitgevallen" = NPULS_ORANJE,
  ## Geslacht
  "man" = NPULS_BLAUW,
  "vrouw" = "#D96EA8",
  ## Opleidingsvorm
  "voltijd" = NPULS_BLAUW,
  "deeltijd" = NPULS_ORANJE,
  ## Int student
  "geen internationale student" = NPULS_BLAUW,
  "internationale student" = NPULS_GROEN,
  ## Sectoren
  "gezondheidszorg" = NPULS_GROEN,
  "economie" = NPULS_BLAUW,
  "gedrag & maatschappij" = NPULS_ORANJE,
  "gedrag en maatschappij" = NPULS_ORANJE,
  "recht" = NPULS_GEEL,
  "natuur" = "#007A5B",
  "techniek" = "#1A3FA0",
  "onderwijs" = NPULS_GROEN,
  "taal & cultuur" = "#B5570D",
  "taal en cultuur" = "#B5570D",
  ## Rendement
  "Diploma binnen 3 jaar" = NPULS_GROEN,
  "Diploma na 3 jaar" = NPULS_GEEL,
  "Diploma binnen 5 jaar" = NPULS_BLAUW,
  "Diploma na 5 jaar" = NPULS_ORANJE,
  "Diploma binnen 5-8 jaar" = "#00796B",
  "Diploma binnen 8 jaar" = "#1A3FA0",
  "Diploma na 8 jaar" = "#B5570D",
  "Geen diploma" = "#D1D5DB",
  "Onbekend" = "#E5E7EB",
  "Onbekend (diplomajaar voor instroomjaar)" = "#E5E7EB",
  ## Uitval
  "Niet uitgevallen" = NPULS_GROEN,
  "Uitgevallen binnen 1 jaar" = NPULS_ORANJE,
  "Uitgevallen in 2e of 3e jaar" = "#DD9E4B",
  "Uitgevallen na 3 jaar" = "#B5570D",
  "Na 1 jaar nog ingeschreven of diploma behaald" = NPULS_GROEN,
  "Na 3 jaar nog ingeschreven of diploma behaald" = NPULS_LICHT_GROEN,
  "Uitgevallen binnen 3 jaar" = "#DD9E4B",
  ## Studiewissel
  "Niet gewisseld" = "#9CA3AF",
  "Gewisseld binnen 1 jaar" = NPULS_GEEL,
  "Gewisseld binnen 3 jaar" = NPULS_GROEN,
  "Gewisseld in het 2e of 3e jaar" = NPULS_GROEN,
  "Niet gewisseld binnen 1 jaar" = "#D1D5DB",
  "Niet gewisseld binnen 3 jaar" = "#E5E7EB",
  "Geen switch bepaald" = "#F3F4F6"
)

## Datadictionary ----

DICT <- data.frame(
  Categorie = c(
    rep("Studentkenmerken", 12),
    rep("Status", 2),
    rep("Rendement", 4),
    rep("Uitval", 4),
    rep("Studiewissel", 11)
  ),
  Kolom = c(
    "inschrijvingsjaar",
    "geslacht",
    "locatie",
    "opleidingscode",
    "opleidingsvorm",
    "opleidingsniveau",
    "int_student",
    "indicatie_EER",
    "sector",
    "leeftijd_bij_instroom",
    "postcode4_student_1okt",
    "postcode4_vooropleiding_voorHO",
    "status",
    "soortdiploma",
    "rendement_3jr",
    "rendement_5jr",
    "rendement_8jr",
    "rendement",
    "uitval_xjr",
    "uitval_1jr",
    "uitval_3jr",
    "uitval",
    "studiewissel_1jr",
    "studiewissel_3jr",
    "opleidingscode_na_switch1jr",
    "opleidingsvorm_na_switch1jr",
    "opleidingsniveau_na_switch1jr",
    "sector_na_switch1jr",
    "opleidingscode_na_switch3jr",
    "opleidingsvorm_na_switch3jr",
    "opleidingsniveau_na_switch3jr",
    "sector_na_switch3jr",
    "studiewissel"
  ),
  Omschrijving = c(
    "Jaar van eerste inschrijving (cohortjaar)",
    "Geslacht van de student",
    "Vestigingslocatie van de opleiding",
    "CROHO-opleidingscode",
    "Opleidingsvorm",
    "Niveau van de opleiding",
    "Indicator internationale student",
    "Indicator EER-student (Europese Economische Ruimte)",
    "Sector van de opleiding (CROHO-onderdeel)",
    "Leeftijd van de student bij aanvang opleiding",
    "Postcode (4-cijferig) van de student op 1 oktober",
    "Postcode (4-cijferig) van vooropleiding voor HO",
    "Status na afloop van de observatieperiode",
    "Soort behaald diploma",
    "Diplomaresultaat binnen 3 jaar na instroom",
    "Diplomaresultaat binnen 5 jaar na instroom",
    "Diplomaresultaat binnen 8 jaar na instroom",
    "Samengevatte rendementsuitkomst",
    "Uitvalstatus binnen een variabele termijn",
    "Uitvalstatus na 1 jaar",
    "Uitvalstatus na 3 jaar",
    "Samengevatte uitvaluitkomst",
    "Studiewissel binnen 1 jaar na instroom",
    "Studiewissel binnen 3 jaar na instroom",
    "Opleidingscode na wissel (1 jaar)",
    "Opleidingsvorm na wissel (1 jaar)",
    "Opleidingsniveau na wissel (1 jaar)",
    "Sector na wissel (1 jaar)",
    "Opleidingscode na wissel (3 jaar)",
    "Opleidingsvorm na wissel (3 jaar)",
    "Opleidingsniveau na wissel (3 jaar)",
    "Sector na wissel (3 jaar)",
    "Samengevatte studiewisseluitkomst"
  ),
  stringsAsFactors = FALSE
)

## Vereiste 1CHO-kolommen ----

VEREISTE_KOLOMMEN <- c(
  "persoonsgebonden_nummer",
  "inschrijvingsjaar",
  "verblijfsjaar_actuele_instelling",
  "diplomajaar",
  "soort_hoger_onderwijs",
  "soort_inschrijving_actuele_instelling",
  "geslacht",
  "opleidingsvorm",
  "indicatie_internationale_student",
  "indicatie_eer_actueel",
  "croho_onderdeel_actuele_opleiding",
  "soort_diploma_instelling",
  "opleiding_actueel_equivalent",
  "type_hoger_onderwijs_binnen_soort_hoger_onderwijs",
  "leeftijd_per_peildatum_1_oktober",
  "postcodecijfers_student_op_1_oktober",
  "postcodecijfers_van_de_hoogste_vooropl_voor_het_ho",
  "opleidingscode_naam_opleiding",
  "vestigingsnummer_gemeentenaam_volgens_rio"
)

## Hulpfuncties ----

kleur_voor <- function(waarden) {
  waarden <- as.character(waarden)
  gevonden <- KLEUREN[waarden]
  ontbrekend <- is.na(gevonden)
  if (any(ontbrekend)) {
    extra <- scales::hue_pal()(sum(ontbrekend))
    gevonden[ontbrekend] <- extra
  }
  names(gevonden) <- waarden
  gevonden
}

thema <- function() {
  theme_minimal(base_size = 12, base_family = FONT_FMLY) +
    theme(
      panel.grid.major.x = element_blank(),
      panel.grid.minor = element_blank(),
      plot.title = element_text(face = "bold", size = 12),
      plot.background = element_rect(fill = "white", color = NA),
      panel.background = element_rect(fill = "white", color = NA),
      axis.text = element_text(color = "#374151"),
      legend.position = "bottom",
      legend.title = element_blank()
    )
}

leeg_plot <- function(bericht = "Geen data voor deze selectie") {
  ggplot() +
    annotate(
      "text",
      x = 0.5,
      y = 0.5,
      label = bericht,
      color = KLEUR_NEUTRAAL,
      size = 5
    ) +
    theme_void()
}

pct_bar <- function(data, x, titel = NULL) {
  if (nrow(data) == 0) {
    return(leeg_plot())
  }
  tel <- data |>
    count(waarde = as.character({{ x }})) |>
    filter(!is.na(waarde)) |>
    mutate(pct = n / sum(n))
  if (nrow(tel) == 0) {
    return(leeg_plot())
  }
  kleuren <- kleur_voor(tel$waarde)
  ggplot(tel, aes(x = fct_reorder(waarde, pct), y = pct, fill = waarde)) +
    geom_col(show.legend = FALSE) +
    geom_text(
      aes(label = paste0(round(pct * 100), "% (n=", n, ")")),
      hjust = -0.05,
      size = 3.3
    ) +
    scale_y_continuous(labels = percent, limits = c(0, 1.2)) +
    scale_fill_manual(values = kleuren, breaks = names(kleuren)) +
    coord_flip() +
    labs(x = NULL, y = NULL, title = titel) +
    thema() +
    theme(legend.position = "none")
}

trend_lijn <- function(
  data,
  y_col,
  kleur_col,
  kleuren_schaal,
  titel,
  y_label = percent
) {
  if (nrow(data) == 0 || nrow(data |> filter(!is.na(.data[[y_col]]))) == 0) {
    return(leeg_plot())
  }
  lijn_data <- data |>
    group_by(.data[[kleur_col]]) |>
    filter(n() >= 2) |>
    ungroup()
  ggplot(
    data,
    aes(x = inschrijvingsjaar, y = .data[[y_col]], color = .data[[kleur_col]])
  ) +
    geom_line(data = lijn_data, linewidth = 1.3) +
    geom_point(size = 3) +
    scale_y_continuous(labels = y_label) +
    scale_x_continuous(breaks = scales::pretty_breaks(n = 7)) +
    scale_color_manual(values = kleuren_schaal) +
    labs(x = "Instroomjaar", y = NULL, title = titel) +
    thema()
}

instroom_trend <- function(data, titel = "Instroom per cohortjaar") {
  if (nrow(data) == 0) {
    return(leeg_plot())
  }
  agg <- data |> count(inschrijvingsjaar)
  if (nrow(agg) == 0) {
    return(leeg_plot())
  }
  lijn_data <- if (nrow(agg) >= 2) agg else agg[0, ]
  ggplot(agg, aes(x = inschrijvingsjaar, y = n)) +
    geom_line(data = lijn_data, color = NPULS_BLAUW, linewidth = 1.3) +
    geom_point(color = NPULS_BLAUW, size = 3) +
    scale_x_continuous(breaks = scales::pretty_breaks(n = 7)) +
    labs(x = "Instroomjaar", y = "Studenten", title = titel) +
    thema()
}

rendement_trend <- function(data, titel = "Rendement per cohortjaar") {
  if (nrow(data) == 0) {
    return(leeg_plot())
  }
  agg <- data |>
    mutate(
      `3 jaar` = rendement_3jr == "Diploma binnen 3 jaar",
      `5 jaar` = rendement_5jr == "Diploma binnen 5 jaar",
      `8 jaar` = rendement_8jr == "Diploma binnen 8 jaar"
    ) |>
    group_by(inschrijvingsjaar) |>
    summarise(
      `3 jaar` = mean(`3 jaar`, na.rm = TRUE),
      `5 jaar` = mean(`5 jaar`, na.rm = TRUE),
      `8 jaar` = mean(`8 jaar`, na.rm = TRUE),
      n = n(),
      .groups = "drop"
    ) |>
    filter(n >= 5) |>
    pivot_longer(
      c(`3 jaar`, `5 jaar`, `8 jaar`),
      names_to = "termijn",
      values_to = "pct"
    ) |>
    filter(!is.na(pct))
  if (nrow(agg) == 0) {
    return(leeg_plot())
  }
  trend_lijn(agg, "pct", "termijn", KLEUREN_TERMIJN, titel)
}

uitval_trend <- function(data, titel = "Uitval per cohortjaar") {
  if (nrow(data) == 0) {
    return(leeg_plot())
  }
  agg <- data |>
    mutate(
      `binnen 1 jaar` = uitval_1jr == "Uitgevallen binnen 1 jaar",
      `binnen 3 jaar` = uitval_3jr == "Uitgevallen binnen 3 jaar"
    ) |>
    group_by(inschrijvingsjaar) |>
    summarise(
      `binnen 1 jaar` = mean(`binnen 1 jaar`, na.rm = TRUE),
      `binnen 3 jaar` = mean(`binnen 3 jaar`, na.rm = TRUE),
      n = n(),
      .groups = "drop"
    ) |>
    filter(n >= 5) |>
    pivot_longer(
      c(`binnen 1 jaar`, `binnen 3 jaar`),
      names_to = "termijn",
      values_to = "pct"
    ) |>
    filter(!is.na(pct))
  if (nrow(agg) == 0) {
    return(leeg_plot())
  }
  trend_lijn(agg, "pct", "termijn", KLEUREN_UITVAL, titel)
}

wissel_trend <- function(data, titel = "Studiewissel per cohortjaar") {
  if (nrow(data) == 0) {
    return(leeg_plot())
  }
  agg <- data |>
    mutate(
      `binnen 1 jaar` = studiewissel_1jr == "Gewisseld binnen 1 jaar",
      `binnen 3 jaar` = studiewissel_3jr == "Gewisseld binnen 3 jaar"
    ) |>
    group_by(inschrijvingsjaar) |>
    summarise(
      `binnen 1 jaar` = mean(`binnen 1 jaar`, na.rm = TRUE),
      `binnen 3 jaar` = mean(`binnen 3 jaar`, na.rm = TRUE),
      n = n(),
      .groups = "drop"
    ) |>
    filter(n >= 5) |>
    pivot_longer(
      c(`binnen 1 jaar`, `binnen 3 jaar`),
      names_to = "termijn",
      values_to = "pct"
    ) |>
    filter(!is.na(pct))
  if (nrow(agg) == 0) {
    return(leeg_plot())
  }
  trend_lijn(agg, "pct", "termijn", KLEUREN_WISSEL, titel)
}

vb <- function(label, waarde, bg = NPULS_BLAUW, fg = NPULS_GEEL) {
  value_box(
    title = label,
    value = waarde,
    theme = value_box_theme(bg = bg, fg = fg)
  )
}

pct_label <- function(data, conditie) {
  if (nrow(data) == 0) {
    return("—")
  }
  paste0(round(mean(conditie(data), na.rm = TRUE) * 100), "%")
}

n_label <- function(n) {
  formatC(n, format = "d", big.mark = ".")
}

## CSS ----

npuls_css <- "
@import url('https://fonts.googleapis.com/css2?family=Plus+Jakarta+Sans:wght@400;600&display=swap');

*, body { font-family: 'Plus Jakarta Sans', sans-serif !important; }

/* Upload scherm */
.upload-achtergrond {
  min-height: 100vh;
  display: flex;
  align-items: center;
  justify-content: center;
  background: #FFFFFF;
  padding: 2rem;
}
.upload-card {
  width: 480px;
  max-width: 100%;
  box-shadow: 0 4px 32px rgba(61,104,236,0.12);
}
.upload-card .card-header {
  background-color: #3D68EC !important;
  color: #F4D74B !important;
  border-bottom: none !important;
  padding: 1.25rem 1.5rem;
}
.upload-card .card-header .npuls-payoff { color: #F4D74B; }
.upload-card .card-body { padding: 1.75rem 1.5rem; }
.upload-verwerk-btn {
  background-color: #3D68EC !important;
  color: #F4D74B !important;
  border: none !important;
  font-weight: 600 !important;
  width: 100%;
  margin-top: 0.5rem;
  padding: 0.65rem;
  font-size: 0.95rem;
}
.upload-verwerk-btn:hover {
  background-color: #2850C8 !important;
}
/* Dashboard header */
.dashboard-header {
  background-color: #F4D9DC;
  border-bottom: 3px solid #DD784B;
  padding: 0.5rem 1.25rem;
  display: flex;
  align-items: center;
  position: sticky;
  top: 0;
  z-index: 100;
}
.dashboard-brand {
  color: #3D68EC;
  font-weight: 600;
  font-size: 1.05rem;
  display: flex;
  flex-direction: column;
  line-height: 1.2;
}

/* Pay-off */
.npuls-payoff {
  display: block;
  color: #DD784B;
  font-size: 0.6rem;
  font-weight: 600;
  text-transform: uppercase;
  letter-spacing: 0.12em;
  margin-top: -0.05rem;
}

/* Sidebar */
.sidebar-title-box {
  color: #3D68EC;
  font-weight: 600;
  font-size: 0.75rem;
  text-transform: uppercase;
  letter-spacing: 0.1em;
  margin-bottom: 0.75rem;
}
.sidebar .form-label,
.sidebar label,
.sidebar .shiny-input-container > label {
  color: #3D68EC !important;
  font-size: 0.75rem;
  font-weight: 600;
  text-transform: uppercase;
  letter-spacing: 0.06em;
}
.sidebar .selectize-input,
.sidebar .form-select,
.sidebar .form-control {
  background-color: #FFFFFF !important;
  border: 1px solid #3D68EC !important;
  color: #000000 !important;
  border-radius: 4px;
}
.sidebar .selectize-dropdown { background-color: #FFFFFF; color: #000000; }
.sidebar .selectize-dropdown .option:hover { background-color: #D6E2FD; }
.sidebar .irs-bar { background: #3D68EC; border-color: #3D68EC; }
.sidebar .irs-single { background: #3D68EC; }
.sidebar .irs-handle { background: #DD784B; border-color: #DD784B; }
.sidebar .irs-line { background: #E8C4C8; }
.sidebar .irs-grid-text { color: #374151; }
.sidebar .irs-min, .sidebar .irs-max { color: #374151; background: transparent; }
.sidebar .form-check-label { color: #000000 !important; font-size: 0.85rem; font-weight: 400; text-transform: none; letter-spacing: 0; }
.sidebar .form-check-input:checked { background-color: #3D68EC; border-color: #3D68EC; }
.sidebar hr { border-color: #E0B4BA !important; }
.sidebar .n-students { color: #374151; font-size: 0.85rem; }

/* Card headers */
.card-header {
  background-color: #D6E2FD !important;
  color: #000000 !important;
  font-weight: 600 !important;
  font-size: 0.78rem !important;
  text-transform: uppercase;
  letter-spacing: 0.06em;
  border-bottom: 2px solid #3D68EC !important;
}

/* Nav tabs */
.nav-tabs .nav-link.active {
  background-color: #3D68EC !important;
  color: #F4D74B !important;
  border-color: #3D68EC !important;
  font-weight: 600;
}
.nav-tabs .nav-link:hover:not(.active) {
  color: #3D68EC;
  border-bottom-color: #3D68EC;
}
.nav-tabs .nav-link { font-weight: 500; font-size: 0.9rem; }

/* Value boxes */
.value-box .value-box-title {
  font-size: 0.72rem;
  text-transform: uppercase;
  letter-spacing: 0.08em;
  opacity: 0.9;
  font-weight: 600;
}
.value-box .value-box-value { font-size: 1.9rem; font-weight: 600; }
"

## UI ----

ui <- page_fluid(
  theme = bs_theme(
    version = 5,
    bg = "#FFFFFF",
    fg = "#000000",
    primary = NPULS_BLAUW,
    success = NPULS_GROEN,
    warning = NPULS_GEEL,
    danger = NPULS_ORANJE,
    base_font = font_google("Plus Jakarta Sans"),
    heading_font = font_google("Plus Jakarta Sans")
  ),
  tags$head(tags$style(npuls_css)),
  style = "padding: 0; margin: 0;",
  uiOutput("scherm")
)

## Server ----

server <- function(input, output, session) {
  fase <- reactiveVal("upload")
  df_data <- reactiveVal(NULL)

  ## Scherm switch ----

  output$scherm <- renderUI({
    if (fase() == "upload") {
      tags$div(
        class = "upload-achtergrond",
        card(
          class = "upload-card",
          card_header(
            tags$div(
              style = "display:flex;align-items:center;gap:0.85rem;",
              HTML(
                '<svg width="32" height="32" viewBox="0 0 32 32">
                  <circle cx="16" cy="16" r="14" fill="none" stroke="#F4D74B" stroke-width="1.5"/>
                  <circle cx="16" cy="16" r="9"  fill="none" stroke="#FFFFFF" stroke-width="1.5"/>
                  <circle cx="16" cy="16" r="4"  fill="#F4D74B"/>
                </svg>'
              ),
              tags$div(
                tags$div(
                  "Staat van Onderwijsinstelling",
                  style = "font-size:1.1rem;font-weight:600;color:#F4D74B;"
                ),
                tags$span("Onderwijs bewegen.", class = "npuls-payoff")
              )
            )
          ),
          card_body(
            fileInput(
              "bestand_upload",
              "1CHO CSV-bestand",
              accept = ".csv",
              buttonLabel = "Bladeren...",
              placeholder = "Geen bestand geselecteerd"
            ),
            uiOutput("btn_verwerk_ui")
          )
        )
      )
    } else {
      req(df_data())
      d <- df_data()
      jaren_d <- sort(unique(d$inschrijvingsjaar))
      locaties_d <- sort(unique(as.character(d$locatie)))
      sectoren_d <- sort(na.omit(unique(as.character(d$sector))))
      niveaus_d <- sort(na.omit(unique(as.character(d$opleidingsniveau))))
      vormen_d <- sort(na.omit(unique(as.character(d$opleidingsvorm))))
      opleidingen_d <- sort(na.omit(unique(as.character(d$opleidingsnaam))))

      tagList(
        tags$div(
          class = "dashboard-header",
          tags$div(
            class = "dashboard-brand",
            "Staat van Onderwijsinstelling",
            tags$span("Onderwijs bewegen.", class = "npuls-payoff")
          ),
          tags$div(
            style = "margin-left:auto;",
            actionButton(
              "btn_opnieuw",
              "Nieuw bestand",
              class = "btn btn-sm",
              style = "background:#FFFFFF;border:1.5px solid #3D68EC;color:#3D68EC;font-size:0.75rem;font-weight:600;"
            )
          )
        ),
        layout_sidebar(
          style = "min-height: calc(100vh - 52px);",
          sidebar = sidebar(
            width = 260,
            bg = NPULS_ROZE,
            fg = NPULS_ZWART,
            tags$div(
              style = "margin-bottom:0.5rem;",
              HTML(
                '<svg width="32" height="32" viewBox="0 0 32 32" style="display:block;margin-bottom:0.5rem">
                  <circle cx="16" cy="16" r="14" fill="none" stroke="#3D68EC" stroke-width="1.5"/>
                  <circle cx="16" cy="16" r="9"  fill="none" stroke="#DD784B" stroke-width="1.5"/>
                  <circle cx="16" cy="16" r="4"  fill="#DD784B"/>
                </svg>'
              ),
              tags$div(class = "sidebar-title-box", "Filters")
            ),
            sliderInput(
              "jaar_filter",
              "Instroomjaar",
              min = min(jaren_d),
              max = max(jaren_d),
              value = c(min(jaren_d), max(jaren_d)),
              sep = "",
              step = 1
            ),
            selectInput(
              "locatie_filter",
              "Locatie",
              choices = c("Alle locaties" = "", locaties_d),
              selected = ""
            ),
            selectInput(
              "sector_filter",
              "Sector",
              choices = c("Alle sectoren" = "", sectoren_d),
              selected = ""
            ),
            selectInput(
              "opleiding_filter",
              "Opleiding",
              choices = c("Alle opleidingen" = "", opleidingen_d),
              selected = ""
            ),
            selectInput(
              "niveau_filter",
              "Opleidingsniveau",
              choices = c("Alle niveaus" = "", niveaus_d),
              selected = ""
            ),
            radioButtons(
              "vorm_filter",
              "Opleidingsvorm",
              choices = c(
                "Alle" = "",
                setNames(vormen_d, tools::toTitleCase(vormen_d))
              ),
              selected = "",
              inline = TRUE
            ),
            radioButtons(
              "geslacht_filter",
              "Geslacht",
              choices = c("Alle" = "", "Man" = "man", "Vrouw" = "vrouw"),
              selected = "",
              inline = TRUE
            ),
            tags$hr(),
            uiOutput("n_label"),
            downloadButton(
              "download_csv",
              "Download als CSV",
              style = "width:100%;margin-top:0.5rem;background:#3D68EC;color:#F4D74B;border:none;font-size:0.78rem;font-weight:600;"
            )
          ),
          navset_card_tab(
            ## Overzicht ----
            nav_panel(
              "Overzicht",
              layout_columns(
                col_widths = c(3, 3, 3, 3),
                uiOutput("kpi_n"),
                uiOutput("kpi_diploma"),
                uiOutput("kpi_uitval_ov"),
                uiOutput("kpi_wissel_ov")
              ),
              layout_columns(
                col_widths = c(4, 4, 4),
                card(
                  card_header("Status"),
                  plotlyOutput("plot_status", height = "260px")
                ),
                card(
                  card_header("Geslacht"),
                  plotlyOutput("plot_ov_geslacht", height = "260px")
                ),
                card(
                  card_header("Opleidingsvorm"),
                  plotlyOutput("plot_ov_vorm", height = "260px")
                )
              ),
              card(
                card_header("Instroom per cohortjaar"),
                plotlyOutput("plot_overzicht_instroom", height = "340px")
              )
            ),

            ## Instroom ----
            nav_panel(
              "Instroom",
              card(
                card_header("Instroom per cohortjaar"),
                plotlyOutput("plot_instroom_trend", height = "360px")
              ),
              layout_columns(
                col_widths = c(4, 4, 4),
                card(
                  card_header("Sector"),
                  plotlyOutput("plot_instroom_sector", height = "280px")
                ),
                card(
                  card_header("Internationale studenten"),
                  plotlyOutput("plot_instroom_int", height = "280px")
                ),
                card(
                  card_header("Geslacht"),
                  plotlyOutput("plot_instroom_geslacht", height = "280px")
                )
              ),
              card(
                card_header("Leeftijdsverdeling bij instroom"),
                plotlyOutput("plot_instroom_leeftijd", height = "260px")
              )
            ),

            ## Rendement ----
            nav_panel(
              "Rendement",
              layout_columns(
                col_widths = c(4, 4, 4),
                uiOutput("kpi_rend3"),
                uiOutput("kpi_rend5"),
                uiOutput("kpi_rend8")
              ),
              card(
                card_header("Rendement per cohortjaar"),
                plotlyOutput("plot_rendement_trend", height = "420px")
              ),
              layout_columns(
                col_widths = c(4, 4, 4),
                card(
                  card_header("Rendement 3 jaar"),
                  plotlyOutput("plot_rend3", height = "240px")
                ),
                card(
                  card_header("Rendement 5 jaar"),
                  plotlyOutput("plot_rend5", height = "240px")
                ),
                card(
                  card_header("Rendement 8 jaar"),
                  plotlyOutput("plot_rend8", height = "240px")
                )
              )
            ),

            ## Uitval ----
            nav_panel(
              "Uitval",
              layout_columns(
                col_widths = c(4, 4, 4),
                uiOutput("kpi_uitval_kpi"),
                uiOutput("kpi_uitval1"),
                uiOutput("kpi_uitval3")
              ),
              card(
                card_header("Uitval per cohortjaar"),
                plotlyOutput("plot_uitval_trend", height = "420px")
              ),
              layout_columns(
                col_widths = c(6, 6),
                card(
                  card_header("Uitval samengevat"),
                  plotlyOutput("plot_uitval_detail", height = "260px")
                ),
                card(
                  card_header("Uitval per sector"),
                  plotlyOutput("plot_uitval_sector", height = "260px")
                )
              )
            ),

            ## Studiewissel ----
            nav_panel(
              "Studiewissel",
              layout_columns(
                col_widths = c(6, 6),
                uiOutput("kpi_wissel1"),
                uiOutput("kpi_wissel3")
              ),
              card(
                card_header("Studiewissel per cohortjaar"),
                plotlyOutput("plot_wissel_trend", height = "420px")
              ),
              layout_columns(
                col_widths = c(4, 4, 4),
                card(
                  card_header("Studiewissel samengevat"),
                  plotlyOutput("plot_wissel_samen", height = "260px")
                ),
                card(
                  card_header("Sector na wissel (1 jaar)"),
                  plotlyOutput("plot_wissel_sector1", height = "260px")
                ),
                card(
                  card_header("Sector na wissel (3 jaar)"),
                  plotlyOutput("plot_wissel_sector3", height = "260px")
                )
              )
            ),

            ## Data ----
            nav_panel(
              "Data",
              card(
                card_header("Datadictionary"),
                DTOutput("tabel_dict")
              ),
              card(
                card_header("Voorbeeld dataset (eerste 20 rijen)"),
                DTOutput("tabel_voorbeeld")
              )
            )
          )
        )
      )
    }
  })

  ## Verwerken ----

  toon_fout <- function(bericht) {
    showNotification(bericht, type = "error", duration = NULL)
  }

  output$btn_verwerk_ui <- renderUI({
    btn <- actionButton(
      "btn_verwerk",
      "Data verwerken",
      class = "upload-verwerk-btn"
    )
    if (is.null(input$bestand_upload)) {
      tagAppendAttributes(
        btn,
        disabled = "disabled",
        style = "opacity: 0.5; cursor: not-allowed;"
      )
    } else {
      btn
    }
  })
  outputOptions(output, "btn_verwerk_ui", suspendWhenHidden = FALSE)

  observeEvent(input$btn_verwerk, {
    if (is.null(input$bestand_upload)) {
      toon_fout("Selecteer eerst een CSV-bestand.")
      return()
    }

    kolomnamen <- names(readr::read_csv2(
      input$bestand_upload$datapath,
      n_max = 0,
      show_col_types = FALSE
    ))

    ontbrekend <- setdiff(VEREISTE_KOLOMMEN, kolomnamen)
    if (length(ontbrekend) > 0) {
      toon_fout(paste0(
        "Ontbrekende kolom(men): ",
        paste(ontbrekend, collapse = ", ")
      ))
      return()
    }

    withProgress(message = "Data wordt verwerkt...", value = 0, {
      tryCatch(
        {
          setProgress(0.10, detail = "Basisbestand inlezen")
          basisbestand <- maak_basisbestand(input$bestand_upload$datapath)
          jaar <- max(basisbestand$inschrijvingsjaar, na.rm = TRUE) + 1L

          soort_ho <- unique(basisbestand$soort_hoger_onderwijs)

          setProgress(0.25, detail = "Instroomcohort aanmaken")
          cohorten <- maak_instroom_cohort(basisbestand, soort_ho)

          setProgress(0.40, detail = "Diploma bepalen")
          diploma <- maak_diploma_behaald(basisbestand)

          setProgress(0.55, detail = "Rendement berekenen")
          rendement <- bereken_rendement(cohorten, diploma)

          setProgress(0.70, detail = "Uitval berekenen")
          uitval <- bereken_uitval(basisbestand, diploma, cohorten, jaar)

          setProgress(0.85, detail = "Studiewissel berekenen")
          wissel <- bereken_studiewissel(
            basisbestand,
            cohorten,
            diploma,
            uitval
          )

          setProgress(0.95, detail = "Combineren")
          result <- combineer_indicatoren(cohorten, rendement, uitval, wissel)

          naam_tabel <- dplyr::distinct(
            basisbestand,
            opleidingscode = as.character(opleiding_actueel_equivalent),
            opleidingsnaam = opleidingscode_naam_opleiding
          )
          result <- dplyr::left_join(
            result,
            naam_tabel,
            by = c("opleidingscode" = "opleidingscode")
          )

          df_data(result)
          fase("dashboard")
        },
        error = function(e) {
          toon_fout(conditionMessage(e))
        }
      )
    })
  })

  ## Terug naar upload ----

  observeEvent(input$btn_opnieuw, {
    df_data(NULL)
    fase("upload")
  })

  ## Gefilterde data ----

  df <- reactive({
    req(fase() == "dashboard", df_data(), !is.null(input$jaar_filter))
    d <- df_data()
    d <- filter(
      d,
      inschrijvingsjaar >= input$jaar_filter[1],
      inschrijvingsjaar <= input$jaar_filter[2]
    )
    if (!is.null(input$locatie_filter) && nchar(input$locatie_filter) > 0) {
      d <- filter(d, as.character(locatie) == input$locatie_filter)
    }
    if (!is.null(input$sector_filter) && nchar(input$sector_filter) > 0) {
      d <- filter(d, as.character(sector) == input$sector_filter)
    }
    if (!is.null(input$opleiding_filter) && nchar(input$opleiding_filter) > 0) {
      d <- filter(d, as.character(opleidingsnaam) == input$opleiding_filter)
    }
    if (!is.null(input$niveau_filter) && nchar(input$niveau_filter) > 0) {
      d <- filter(d, as.character(opleidingsniveau) == input$niveau_filter)
    }
    if (!is.null(input$vorm_filter) && nchar(input$vorm_filter) > 0) {
      d <- filter(d, as.character(opleidingsvorm) == input$vorm_filter)
    }
    if (!is.null(input$geslacht_filter) && nchar(input$geslacht_filter) > 0) {
      d <- filter(d, as.character(geslacht) == input$geslacht_filter)
    }
    d
  })

  ## Sidebar teller ----

  output$n_label <- renderUI({
    tags$p(tags$strong(n_label(nrow(df()))), " studenten", class = "n-students")
  })

  ## KPI's overzicht ----

  output$kpi_n <- renderUI(vb(
    "Studenten",
    n_label(nrow(df())),
    bg = NPULS_BLAUW,
    fg = NPULS_GEEL
  ))
  output$kpi_diploma <- renderUI(vb(
    "Diploma behaald",
    pct_label(df(), \(d) d$status == "Diploma behaald"),
    bg = NPULS_GROEN,
    fg = NPULS_ZWART
  ))
  output$kpi_uitval_ov <- renderUI(vb(
    "Uitgevallen",
    pct_label(df(), \(d) d$status == "Uitgevallen"),
    bg = NPULS_ORANJE,
    fg = NPULS_ZWART
  ))
  output$kpi_wissel_ov <- renderUI(vb(
    "Gewisseld binnen 1 jaar",
    pct_label(df(), \(d) d$studiewissel_1jr == "Gewisseld binnen 1 jaar"),
    bg = NPULS_GEEL,
    fg = NPULS_ZWART
  ))

  ## KPI's rendement ----

  output$kpi_rend3 <- renderUI(vb(
    "Diploma binnen 3 jaar",
    pct_label(df(), \(d) d$rendement_3jr == "Diploma binnen 3 jaar"),
    bg = NPULS_ORANJE,
    fg = NPULS_ZWART
  ))
  output$kpi_rend5 <- renderUI(vb(
    "Diploma binnen 5 jaar",
    pct_label(df(), \(d) d$rendement_5jr == "Diploma binnen 5 jaar"),
    bg = NPULS_BLAUW,
    fg = NPULS_GEEL
  ))
  output$kpi_rend8 <- renderUI(vb(
    "Diploma binnen 8 jaar",
    pct_label(df(), \(d) d$rendement_8jr == "Diploma binnen 8 jaar"),
    bg = NPULS_GROEN,
    fg = NPULS_ZWART
  ))

  ## KPI's uitval ----

  output$kpi_uitval_kpi <- renderUI(vb(
    "% uitgevallen",
    pct_label(df(), \(d) d$status == "Uitgevallen"),
    bg = NPULS_ORANJE,
    fg = NPULS_ZWART
  ))
  output$kpi_uitval1 <- renderUI(vb(
    "Uitval binnen 1 jaar",
    pct_label(df(), \(d) d$uitval_1jr == "Uitgevallen binnen 1 jaar"),
    bg = NPULS_ORANJE,
    fg = NPULS_ZWART
  ))
  output$kpi_uitval3 <- renderUI(vb(
    "Uitval binnen 3 jaar",
    pct_label(df(), \(d) d$uitval_3jr == "Uitgevallen binnen 3 jaar"),
    bg = NPULS_BLAUW,
    fg = NPULS_GEEL
  ))

  ## KPI's studiewissel ----

  output$kpi_wissel1 <- renderUI(vb(
    "Gewisseld binnen 1 jaar",
    pct_label(df(), \(d) d$studiewissel_1jr == "Gewisseld binnen 1 jaar"),
    bg = NPULS_GEEL,
    fg = NPULS_ZWART
  ))
  output$kpi_wissel3 <- renderUI(vb(
    "Gewisseld binnen 3 jaar",
    pct_label(df(), \(d) d$studiewissel_3jr == "Gewisseld binnen 3 jaar"),
    bg = NPULS_GROEN,
    fg = NPULS_ZWART
  ))

  ## Plots overzicht ----

  output$plot_status <- renderPlotly(ggplotly(pct_bar(df(), status)))
  output$plot_ov_geslacht <- renderPlotly(ggplotly(pct_bar(df(), geslacht)))
  output$plot_ov_vorm <- renderPlotly(ggplotly(pct_bar(df(), opleidingsvorm)))
  output$plot_overzicht_instroom <- renderPlotly(
    ggplotly(instroom_trend(df()), tooltip = c("x", "y")) |>
      layout(
        legend = list(
          x = 0.01,
          y = 0.99,
          xanchor = "left",
          yanchor = "top",
          bgcolor = "rgba(255,255,255,0.85)",
          bordercolor = "#D6E2FD",
          borderwidth = 1,
          font = list(size = 11)
        )
      )
  )

  ## Plots instroom ----

  output$plot_instroom_trend <- renderPlotly(
    ggplotly(instroom_trend(df()), tooltip = c("x", "y")) |>
      layout(
        legend = list(
          x = 0.01,
          y = 0.99,
          xanchor = "left",
          yanchor = "top",
          bgcolor = "rgba(255,255,255,0.85)",
          bordercolor = "#D6E2FD",
          borderwidth = 1,
          font = list(size = 11)
        )
      )
  )
  output$plot_instroom_sector <- renderPlotly(ggplotly(pct_bar(
    df(),
    sector
  )))
  output$plot_instroom_int <- renderPlotly(ggplotly(pct_bar(df(), int_student)))
  output$plot_instroom_geslacht <- renderPlotly(ggplotly(pct_bar(
    df(),
    geslacht
  )))
  output$plot_instroom_leeftijd <- renderPlotly({
    d <- df()
    if (nrow(d) == 0) {
      return(ggplotly(leeg_plot()))
    }
    leeftijd <- as.integer(d$leeftijd_bij_instroom)
    leeftijd <- leeftijd[!is.na(leeftijd) & leeftijd > 0 & leeftijd < 80]
    if (length(leeftijd) == 0) {
      return(ggplotly(leeg_plot()))
    }
    p <- ggplot(data.frame(leeftijd = leeftijd), aes(x = leeftijd)) +
      geom_histogram(
        fill = NPULS_BLAUW,
        alpha = 0.85,
        binwidth = 1,
        boundary = 0
      ) +
      scale_x_continuous(breaks = scales::pretty_breaks(n = 8)) +
      labs(x = "Leeftijd bij instroom", y = "Studenten") +
      thema() +
      theme(
        panel.grid.major.y = element_line(color = "#F3F4F6"),
        panel.grid.major.x = element_blank()
      )
    ggplotly(p) |> layout(showlegend = FALSE)
  })

  ## Plots rendement ----

  output$plot_rendement_trend <- renderPlotly(
    ggplotly(rendement_trend(df()), tooltip = c("x", "y", "colour")) |>
      layout(
        legend = list(
          x = 0.01,
          y = 0.99,
          xanchor = "left",
          yanchor = "top",
          bgcolor = "rgba(255,255,255,0.85)",
          bordercolor = "#D6E2FD",
          borderwidth = 1,
          font = list(size = 11)
        )
      )
  )
  output$plot_rend3 <- renderPlotly(ggplotly(pct_bar(
    df(),
    rendement_3jr,
    "Rendement 3 jaar"
  )))
  output$plot_rend5 <- renderPlotly(ggplotly(pct_bar(
    df(),
    rendement_5jr,
    "Rendement 5 jaar"
  )))
  output$plot_rend8 <- renderPlotly(ggplotly(pct_bar(
    df(),
    rendement_8jr,
    "Rendement 8 jaar"
  )))

  ## Plots uitval ----

  output$plot_uitval_trend <- renderPlotly(
    ggplotly(uitval_trend(df()), tooltip = c("x", "y", "colour")) |>
      layout(
        legend = list(
          x = 0.01,
          y = 0.99,
          xanchor = "left",
          yanchor = "top",
          bgcolor = "rgba(255,255,255,0.85)",
          bordercolor = "#D6E2FD",
          borderwidth = 1,
          font = list(size = 11)
        )
      )
  )
  output$plot_uitval_detail <- renderPlotly(ggplotly(pct_bar(df(), uitval)))
  output$plot_uitval_sector <- renderPlotly({
    d <- df() |> filter(!is.na(sector))
    if (nrow(d) == 0) {
      return(ggplotly(leeg_plot()))
    }
    agg <- d |>
      group_by(sector = as.character(sector)) |>
      summarise(
        pct = mean(status == "Uitgevallen", na.rm = TRUE),
        .groups = "drop"
      )
    kleuren <- kleur_voor(agg$sector)
    p <- ggplot(
      agg,
      aes(x = fct_reorder(sector, pct), y = pct, fill = sector)
    ) +
      geom_col(show.legend = FALSE) +
      geom_text(
        aes(label = paste0(round(pct * 100), "%")),
        hjust = -0.1,
        size = 3.3
      ) +
      scale_y_continuous(labels = percent, limits = c(0, 1.1)) +
      scale_fill_manual(values = kleuren, breaks = names(kleuren)) +
      coord_flip() +
      labs(x = NULL, y = NULL, title = "% uitgevallen per sector") +
      thema()
    ggplotly(p) |> layout(showlegend = FALSE)
  })

  ## Plots studiewissel ----

  output$plot_wissel_trend <- renderPlotly(
    ggplotly(wissel_trend(df()), tooltip = c("x", "y", "colour")) |>
      layout(
        legend = list(
          x = 0.01,
          y = 0.99,
          xanchor = "left",
          yanchor = "top",
          bgcolor = "rgba(255,255,255,0.85)",
          bordercolor = "#D6E2FD",
          borderwidth = 1,
          font = list(size = 11)
        )
      )
  )
  output$plot_wissel_samen <- renderPlotly(ggplotly(pct_bar(
    df(),
    studiewissel
  )))
  output$plot_wissel_sector1 <- renderPlotly(ggplotly(pct_bar(
    df(),
    sector_na_switch1jr
  )))
  output$plot_wissel_sector3 <- renderPlotly(ggplotly(pct_bar(
    df(),
    sector_na_switch3jr
  )))

  ## Data tab ----

  output$tabel_dict <- renderDT({
    datatable(
      DICT,
      rownames = FALSE,
      options = list(pageLength = 33, dom = "t", ordering = FALSE),
      class = "compact stripe"
    ) |>
      formatStyle(
        "Categorie",
        backgroundColor = styleEqual(
          c(
            "Studentkenmerken",
            "Status",
            "Rendement",
            "Uitval",
            "Studiewissel"
          ),
          c("#CCEEE6", "#D6E2FD", "#FFE4D8", "#FFE4D8", "#FCF5D4")
        ),
        fontWeight = "600"
      )
  })

  output$tabel_voorbeeld <- renderDT({
    req(df_data())
    datatable(
      head(df_data(), 20),
      rownames = FALSE,
      options = list(scrollX = TRUE, pageLength = 10, dom = "tp"),
      class = "compact"
    )
  })

  output$download_csv <- downloadHandler(
    filename = function() {
      paste0("Indicatoren_1cHO_", Sys.Date(), ".csv")
    },
    content = function(file) {
      write_csv(df_data(), file)
    }
  )
}

shinyApp(ui, server)

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
library(staat1cho)

## ----basisbestand-------------------------------------------------------------
library(tibble)

## Drie studenten, twee cohortjaren
## S1: behaalt diploma in 2022 (binnen 3 jaar na instroom 2020)
## S2: valt uit na 1 jaar (verschijnt alleen in 2020)
## S3: wisselt opleiding na 1 jaar (andere opleidingscode in 2021)

hoofd <- "hoofdinschrijving binnen het domein actuele instelling"
bachelor_diploma <- "Hoofd-bachelor-diploma binnen de actuele instelling"

basisbestand <- tibble(
  persoonsgebonden_nummer = c("S1", "S1", "S2", "S3", "S3"),
  inschrijvingsjaar       = c(2020L, 2021L, 2020L, 2020L, 2021L),
  verblijfsjaar_actuele_instelling = c(1L, 2L, 1L, 1L, 2L),
  soort_hoger_onderwijs   = "hbo",
  soort_inschrijving_actuele_instelling = hoofd,
  soort_inschrijving_actuele_instelling_label = hoofd,
  diplomajaar             = c(NA_integer_, 2022L, NA_integer_, NA_integer_, NA_integer_),
  soort_diploma_instelling = c(NA, bachelor_diploma, NA, NA, NA),
  soort_diploma_instelling_label = c(NA, bachelor_diploma, NA, NA, NA),
  geslacht_label = "man",
  opleidingsvorm = "voltijd",
  opleidingsvorm_label = "voltijd",
  indicatie_internationale_student_label = "geen internationale student",
  indicatie_eer_actueel_label = "geen EER-student",
  croho_onderdeel_actuele_opleiding = "techniek",
  croho_onderdeel_actuele_opleiding_label = "techniek",
  opleiding_actueel_equivalent = c("34401", "34401", "34401", "34401", "39999"),
  type_hoger_onderwijs_binnen_soort_hoger_onderwijs = "ba",
  leeftijd_per_peildatum_1_oktober = 19L,
  postcodecijfers_student_op_1_oktober = "4818",
  postcodecijfers_van_de_hoogste_vooropl_voor_het_ho = "4818",
  locatie_label = "Breda"
)

## ----instroom-----------------------------------------------------------------
cohorten <- maak_instroom_cohort(basisbestand, soort_ho = "hbo")
cohorten[, c("persoonsgebonden_nummer", "inschrijvingsjaar", "eerstejaar_instelling")]

## ----diploma------------------------------------------------------------------
diploma <- maak_diploma_behaald(basisbestand)
diploma

## ----rendement----------------------------------------------------------------
rendement <- bereken_rendement(cohorten, diploma)
rendement[, c("persoonsgebonden_nummer", "rendement_3jr", "rendement_5jr", "rendement_8jr")]

## ----uitval-------------------------------------------------------------------
## peiljaar = het jaar na het laatste jaar in de data
uitval <- bereken_uitval(basisbestand, diploma, cohorten, jaar = 2023L)
uitval[, c("persoonsgebonden_nummer", "status", "uitval_1jr", "uitval_3jr")]

## ----wissel-------------------------------------------------------------------
wissel <- bereken_studiewissel(basisbestand, cohorten, diploma, uitval)
wissel[, c("persoonsgebonden_nummer", "studiewissel_1jr", "studiewissel_3jr")]

## ----combineer, warning = FALSE-----------------------------------------------
indicatoren <- combineer_indicatoren(cohorten, rendement, uitval, wissel)
indicatoren[, c("inschrijvingsjaar", "opleidingscode", "status", "rendement", "uitval", "studiewissel")]

## ----dashboard, eval = FALSE--------------------------------------------------
# start_dashboard()

#' Bereken uitvalindicatoren per cohort
#'
#' Bepaalt voor elke student in het instroomcohort of zij zijn uitgevallen,
#' zittend of gediplomeerd. Uitval wordt gemarkeerd als een student niet meer
#' ingeschreven staat en geen diploma heeft.
#'
#' @param basisbestand Tibble zoals gemaakt door [maak_basisbestand()]
#' @param diploma_behaald Tibble zoals gemaakt door [maak_diploma_behaald()]
#' @param cohorten_instroom Tibble zoals gemaakt door [maak_instroom_cohort()]
#' @param jaar Integer, peiljaar van de analyse (bijv. `2025`). Studenten die
#'   in `jaar - 1` nog ingeschreven staan, gelden als zittend.
#'
#' @return Een tibble met kolommen `persoonsgebonden_nummer`,
#'   `laatste_jaar_inschrijving`, `diploma`, `status` (factor:
#'   Diploma behaald / Zittend / Uitgevallen), `uitval_xjr` (jaar van uitval
#'   t.o.v. instroomjaar), `uitval_1jr` en `uitval_3jr` (factoren).
#'   Gooit een fout bij dubbele studenten of ontbrekende statussen.
#'
#' @examples
#' basis <- tibble::tibble(
#'   persoonsgebonden_nummer = c("S1", "S1", "S2"),
#'   inschrijvingsjaar = c(2020L, 2021L, 2020L),
#'   soort_inschrijving_actuele_instelling = "hoofdinschrijving"
#' )
#' diploma <- tibble::tibble(
#'   persoonsgebonden_nummer = "S1",
#'   jaar_eerste_diploma = 2022L,
#'   verblijfsjaar_eerste_diploma = 3L,
#'   diploma = "Diploma behaald (excl. propedeuse)"
#' )
#' cohort <- tibble::tibble(
#'   persoonsgebonden_nummer = c("S1", "S2"),
#'   eerstejaar_instelling = 2020L
#' )
#' bereken_uitval(basis, diploma, cohort, jaar = 2023L)
#' @export
bereken_uitval <- function(
  basisbestand,
  diploma_behaald,
  cohorten_instroom,
  jaar
) {
  ## Bepaal het laatste inschrijvingsjaar per student
  uitstroom <- basisbestand |>
    dplyr::group_by(persoonsgebonden_nummer) |>
    dplyr::arrange(
      dplyr::desc(inschrijvingsjaar),
      soort_inschrijving_actuele_instelling,
      .by_group = TRUE
    ) |>
    dplyr::distinct(persoonsgebonden_nummer, .keep_all = TRUE) |>
    dplyr::ungroup() |>
    dplyr::mutate(
      ## jaar - 1 is het meest recente studiejaar in de data; studenten die
      ## daar nog in staan zijn niet uitgevallen maar zittend
      laatste_jaar_inschrijving = dplyr::case_when(
        inschrijvingsjaar == jaar - 1 ~ NA_real_,
        is.na(inschrijvingsjaar) ~ NA_real_,
        TRUE ~ as.numeric(inschrijvingsjaar)
      )
    ) |>
    dplyr::select(
      persoonsgebonden_nummer,
      inschrijvingsjaar,
      laatste_jaar_inschrijving
    )

  if (any(duplicated(uitstroom$persoonsgebonden_nummer))) {
    rlang::abort("Dubbele studentnummers in het uitvalbestand gevonden!")
  }

  uitval <- uitstroom |>
    dplyr::left_join(diploma_behaald, by = "persoonsgebonden_nummer") |>
    dplyr::left_join(
      cohorten_instroom |>
        dplyr::select(persoonsgebonden_nummer, eerstejaar_instelling),
      by = "persoonsgebonden_nummer"
    ) |>
    dplyr::mutate(
      status = factor(dplyr::case_when(
        diploma == "Diploma behaald (excl. propedeuse)" ~ "Diploma behaald",
        inschrijvingsjaar == jaar - 1 ~ "Zittend",
        TRUE ~ "Uitgevallen"
      )),
      uitval_xjr = dplyr::case_when(
        status == "Uitgevallen" ~ laatste_jaar_inschrijving +
          1 -
          eerstejaar_instelling
      ),
      uitval_1jr = factor(dplyr::case_when(
        uitval_xjr == 1 ~ "Uitgevallen binnen 1 jaar",
        TRUE ~ "Na 1 jaar nog ingeschreven of diploma behaald"
      )),
      uitval_3jr = factor(dplyr::case_when(
        uitval_xjr <= 3 ~ "Uitgevallen binnen 3 jaar",
        TRUE ~ "Na 3 jaar nog ingeschreven of diploma behaald"
      ))
    ) |>
    dplyr::select(
      persoonsgebonden_nummer,
      laatste_jaar_inschrijving,
      diploma,
      status,
      dplyr::starts_with("uitval")
    )

  if (any(is.na(uitval$status))) {
    rlang::abort("Niet alle statussen zijn gevuld")
  }

  uitval
}

maak_basis <- function(pgn, inschrijvingsjaar) {
  tibble(
    persoonsgebonden_nummer = pgn,
    inschrijvingsjaar = inschrijvingsjaar,
    soort_inschrijving_actuele_instelling = "hoofdinschrijving"
  )
}

maak_cohort <- function(pgn, eerstejaar) {
  tibble(
    persoonsgebonden_nummer = pgn,
    eerstejaar_instelling = eerstejaar
  )
}

geen_diplomas <- tibble(
  persoonsgebonden_nummer = character(0),
  jaar_eerste_diploma = integer(0),
  verblijfsjaar_eerste_diploma = integer(0),
  diploma = character(0)
)

met_diploma <- function(pgn) {
  tibble(
    persoonsgebonden_nummer = pgn,
    jaar_eerste_diploma = 2022,
    verblijfsjaar_eerste_diploma = 3L,
    diploma = "Diploma behaald (excl. propedeuse)"
  )
}

## --- status ---

test_that("markeert zittende student correct", {
  jaar <- 2025

  result <- bereken_uitval(
    maak_basis("A", jaar - 1),
    geen_diplomas,
    maak_cohort("A", 2022),
    jaar
  )

  expect_equal(as.character(result$status), "Zittend")
})

test_that("markeert gediplomeerde student correct", {
  jaar <- 2025

  result <- bereken_uitval(
    maak_basis("A", 2022),
    met_diploma("A"),
    maak_cohort("A", 2020),
    jaar
  )

  expect_equal(as.character(result$status), "Diploma behaald")
})

test_that("markeert uitgevallen student correct", {
  jaar <- 2025

  result <- bereken_uitval(
    maak_basis("A", 2021),
    geen_diplomas,
    maak_cohort("A", 2020),
    jaar
  )

  expect_equal(as.character(result$status), "Uitgevallen")
})

test_that("diploma heeft voorrang boven zittend als student ook ingeschreven is", {
  ## Student staat in jaar - 1 ingeschreven maar heeft ook een diploma
  jaar <- 2025

  result <- bereken_uitval(
    maak_basis("A", jaar - 1),
    met_diploma("A"),
    maak_cohort("A", 2020),
    jaar
  )

  expect_equal(as.character(result$status), "Diploma behaald")
})

test_that("status is een factor", {
  jaar <- 2025

  result <- bereken_uitval(
    maak_basis("A", 2021),
    geen_diplomas,
    maak_cohort("A", 2020),
    jaar
  )

  expect_true(is.factor(result$status))
})

## --- uitval_xjr ---

test_that("berekent uitval_xjr correct", {
  jaar <- 2025

  result <- bereken_uitval(
    maak_basis("A", 2021),
    geen_diplomas,
    maak_cohort("A", 2020),
    jaar
  )

  ## 2021 + 1 - 2020 = 2
  expect_equal(result$uitval_xjr, 2)
})

test_that("uitval_xjr is NA voor zittende student", {
  jaar <- 2025

  result <- bereken_uitval(
    maak_basis("A", jaar - 1),
    geen_diplomas,
    maak_cohort("A", 2022),
    jaar
  )

  expect_true(is.na(result$uitval_xjr))
})

test_that("uitval_xjr is NA voor gediplomeerde student", {
  jaar <- 2025

  result <- bereken_uitval(
    maak_basis("A", 2022),
    met_diploma("A"),
    maak_cohort("A", 2020),
    jaar
  )

  expect_true(is.na(result$uitval_xjr))
})

## --- uitval_1jr en uitval_3jr ---

test_that("categoriseert uitval_1jr correct", {
  jaar <- 2025
  basisbestand <- tibble(
    persoonsgebonden_nummer = c("A", "B"),
    inschrijvingsjaar = c(2020, 2021),
    soort_inschrijving_actuele_instelling = "hoofdinschrijving"
  )
  cohorten_instroom <- tibble(
    persoonsgebonden_nummer = c("A", "B"),
    eerstejaar_instelling = 2020
  )

  result <- bereken_uitval(basisbestand, geen_diplomas, cohorten_instroom, jaar)

  ## A: xjr = 1 -> uitgevallen binnen 1 jaar
  expect_equal(
    as.character(result$uitval_1jr[result$persoonsgebonden_nummer == "A"]),
    "Uitgevallen binnen 1 jaar"
  )
  ## B: xjr = 2 -> niet binnen 1 jaar
  expect_equal(
    as.character(result$uitval_1jr[result$persoonsgebonden_nummer == "B"]),
    "Na 1 jaar nog ingeschreven of diploma behaald"
  )
})

test_that("categoriseert uitval_3jr correct inclusief grenswaarde", {
  jaar <- 2025
  basisbestand <- tibble(
    persoonsgebonden_nummer = c("A", "B", "C"),
    ## A: xjr = 2020 + 1 - 2020 = 1, B: xjr = 2022 + 1 - 2020 = 3, C: xjr = 2015 + 1 - 2010 = 6
    inschrijvingsjaar = c(2020, 2022, 2015),
    soort_inschrijving_actuele_instelling = "hoofdinschrijving"
  )
  cohorten_instroom <- tibble(
    persoonsgebonden_nummer = c("A", "B", "C"),
    eerstejaar_instelling = c(2020, 2020, 2010)
  )

  result <- bereken_uitval(basisbestand, geen_diplomas, cohorten_instroom, jaar)

  ## A: xjr = 1 -> uitgevallen binnen 3 jaar
  expect_equal(
    as.character(result$uitval_3jr[result$persoonsgebonden_nummer == "A"]),
    "Uitgevallen binnen 3 jaar"
  )
  ## B: xjr = 3 (exact op grens) -> uitgevallen binnen 3 jaar
  expect_equal(
    as.character(result$uitval_3jr[result$persoonsgebonden_nummer == "B"]),
    "Uitgevallen binnen 3 jaar"
  )
  ## C: xjr = 6 -> na 3 jaar
  expect_equal(
    as.character(result$uitval_3jr[result$persoonsgebonden_nummer == "C"]),
    "Na 3 jaar nog ingeschreven of diploma behaald"
  )
})

test_that("verwerkt meerdere studenten tegelijk correct", {
  jaar <- 2025
  basisbestand <- tibble(
    persoonsgebonden_nummer = c("A", "B", "C"),
    inschrijvingsjaar = c(jaar - 1, 2022, 2021),
    soort_inschrijving_actuele_instelling = "hoofdinschrijving"
  )
  cohorten_instroom <- tibble(
    persoonsgebonden_nummer = c("A", "B", "C"),
    eerstejaar_instelling = 2020
  )

  result <- bereken_uitval(
    basisbestand,
    met_diploma("B"),
    cohorten_instroom,
    jaar
  )

  statussen <- setNames(
    as.character(result$status),
    result$persoonsgebonden_nummer
  )
  expect_equal(statussen[["A"]], "Zittend")
  expect_equal(statussen[["B"]], "Diploma behaald")
  expect_equal(statussen[["C"]], "Uitgevallen")
})

test_that("behoudt student in uitval die niet in cohorten_instroom zit", {
  ## basisbestand bevat meer studenten dan cohorten_instroom
  jaar <- 2025
  basisbestand <- tibble(
    persoonsgebonden_nummer = c("A", "B"),
    inschrijvingsjaar = c(2021, 2021),
    soort_inschrijving_actuele_instelling = "hoofdinschrijving"
  )
  ## Alleen A in cohort
  cohorten_instroom <- maak_cohort("A", 2020)

  result <- bereken_uitval(basisbestand, geen_diplomas, cohorten_instroom, jaar)

  expect_equal(nrow(result), 2)
  ## B heeft geen eerstejaar_instelling (NA na left_join) -> uitval_xjr wordt NA
  expect_true(is.na(result$uitval_xjr[result$persoonsgebonden_nummer == "B"]))
})

#' staat1cho: Studie-indicatoren op basis van 1CHO-data
#'
#' Berekent instroom-, rendement-, uitval- en studiewisselindicatoren
#' vanuit de 1CHO-aanlevering van DUO.
#'
#' @importFrom rlang .data :=
#' @name staat1cho-package
#' @keywords internal
"_PACKAGE"

utils::globalVariables(c(
  "croho_onderdeel_actuele_opleiding",
  "croho_onderdeel_actuele_opleiding_label",
  "diploma",
  "diploma_een_drie",
  "diplomajaar",
  "eerstejaar_instelling",
  "geslacht",
  "geslacht_label",
  "indicatie_eer_actueel",
  "indicatie_eer_actueel_label",
  "indicatie_internationale_student",
  "indicatie_internationale_student_label",
  "inschrijvingsjaar",
  "jaar_eerste_diploma",
  "laatste_jaar_inschrijving",
  "leeftijd_per_peildatum_1_oktober",
  "locatie",
  "locatie_label",
  "opl_dubbel",
  "opleiding_actueel_equivalent",
  "opleidingscode",
  "opleidingsniveau",
  "opleidingsvorm",
  "opleidingsvorm_label",
  "persnr_dubbel",
  "persoonsgebonden_nummer",
  "postcode4_student_1okt",
  "postcode4_vooropleiding_voorHO",
  "postcodecijfers_student_op_1_oktober",
  "postcodecijfers_van_de_hoogste_vooropl_voor_het_ho",
  "rendement_3jr",
  "rendement_8jr",
  "sector",
  "sector_na_switch3jr",
  "soort_diploma_instelling",
  "soort_diploma_instelling_label",
  "soort_hoger_onderwijs",
  "soort_inschrijving_actuele_instelling",
  "soort_inschrijving_actuele_instelling_label",
  "status",
  "studiewissel_1jr",
  "studiewissel_3jr",
  "type_hoger_onderwijs_binnen_soort_hoger_onderwijs",
  "uitval_een_drie",
  "uitval_xjr",
  "verblijfsjaar_actuele_instelling",
  "verblijfsjaar_eerste_diploma",
  "verschil_kalenderjaren",
  "vestigingsnummer_gemeentenaam_volgens_rio"
))

#' Combineer alle indicatoren tot een analysebestand
#'
#' Voegt rendement-, uitval- en studiewisselindicatoren samen met het
#' instroomcohort. Past kolomnamen en factorniveaus aan voor gebruik in
#' rapportages.
#'
#' @param cohorten_instroom Tibble zoals gemaakt door [maak_instroom_cohort()]
#' @param rendement_indicatoren Tibble zoals gemaakt door [bereken_rendement()]
#' @param uitval_indicatoren Tibble zoals gemaakt door [bereken_uitval()]
#' @param studiewissel_indicatoren Tibble zoals gemaakt door
#'   [bereken_studiewissel()]
#'
#' @return Een tibble met een rij per student en gecombineerde indicator-
#'   kolommen, klaar voor rapportage. Bevat o.a. `status`, `rendement`,
#'   `uitval`, `studiewissel` en alle onderliggende deelscores.
#'
#' @examples
#' cohort <- tibble::tibble(
#'   persoonsgebonden_nummer = "S1",
#'   inschrijvingsjaar = 2020L,
#'   eerstejaar_instelling = 2020L,
#'   geslacht_label = "man",
#'   locatie_label = "Breda",
#'   opleiding_actueel_equivalent = "34401",
#'   opleidingsvorm_label = "voltijd",
#'   type_hoger_onderwijs_binnen_soort_hoger_onderwijs = "ba",
#'   indicatie_internationale_student_label = "geen internationale student",
#'   indicatie_eer_actueel_label = "geen EER-student",
#'   croho_onderdeel_actuele_opleiding_label = "techniek",
#'   leeftijd_per_peildatum_1_oktober = 19L,
#'   postcodecijfers_student_op_1_oktober = "4818",
#'   postcodecijfers_van_de_hoogste_vooropl_voor_het_ho = "4818",
#'   soort_diploma_instelling_label = NA_character_
#' )
#' rendement <- tibble::tibble(
#'   persoonsgebonden_nummer = "S1",
#'   eerstejaar_instelling = 2020L,
#'   jaar_eerste_diploma = NA_real_,
#'   verblijfsjaar_eerste_diploma = NA_integer_,
#'   diploma = NA_character_,
#'   rendement_xjaar = factor(NA_character_),
#'   rendement_3jr = factor("Geen diploma"),
#'   rendement_5jr = factor("Geen diploma"),
#'   rendement_8jr = factor("Geen diploma")
#' )
#' uitval <- tibble::tibble(
#'   persoonsgebonden_nummer = "S1",
#'   laatste_jaar_inschrijving = NA_real_,
#'   diploma = NA_character_,
#'   status = factor("Zittend"),
#'   uitval_xjr = NA_real_,
#'   uitval_1jr = factor("Na 1 jaar nog ingeschreven of diploma behaald"),
#'   uitval_3jr = factor("Na 3 jaar nog ingeschreven of diploma behaald")
#' )
#' wissel <- tibble::tibble(
#'   persoonsgebonden_nummer = "S1",
#'   studiewissel_1jr = factor("Niet gewisseld binnen 1 jaar"),
#'   studiewissel_3jr = factor("Niet gewisseld binnen 3 jaar"),
#'   opleidingscode_na_switch1jr = factor(NA_character_),
#'   opleidingsvorm_na_switch1jr = factor(NA_character_),
#'   opleidingsniveau_na_switch1jr = factor(NA_character_),
#'   sector_na_switch1jr = factor(NA_character_),
#'   opleidingscode_na_switch3jr = factor(NA_character_),
#'   opleidingsvorm_na_switch3jr = factor(NA_character_),
#'   opleidingsniveau_na_switch3jr = factor(NA_character_),
#'   sector_na_switch3jr = factor(NA_character_)
#' )
#' suppressWarnings(combineer_indicatoren(cohort, rendement, uitval, wissel))
#' @export
combineer_indicatoren <- function(
  cohorten_instroom,
  rendement_indicatoren,
  uitval_indicatoren,
  studiewissel_indicatoren
) {
  cohorten_instroom |>
    dplyr::left_join(rendement_indicatoren, by = "persoonsgebonden_nummer") |>
    dplyr::left_join(uitval_indicatoren, by = "persoonsgebonden_nummer") |>
    dplyr::left_join(
      studiewissel_indicatoren,
      by = "persoonsgebonden_nummer"
    ) |>

    dplyr::select(
      inschrijvingsjaar,
      geslacht = geslacht_label,
      locatie = locatie_label,
      opleidingscode = opleiding_actueel_equivalent,
      opleidingsvorm = opleidingsvorm_label,
      opleidingsniveau = type_hoger_onderwijs_binnen_soort_hoger_onderwijs,
      int_student = indicatie_internationale_student_label,
      indicatie_EER = indicatie_eer_actueel_label,
      sector = croho_onderdeel_actuele_opleiding_label,
      leeftijd_bij_instroom = leeftijd_per_peildatum_1_oktober,
      postcode4_student_1okt = postcodecijfers_student_op_1_oktober,
      postcode4_vooropleiding_voorHO = postcodecijfers_van_de_hoogste_vooropl_voor_het_ho,
      status,
      soortdiploma = soort_diploma_instelling_label,
      rendement_3jr:rendement_8jr,
      uitval_xjr:sector_na_switch3jr
    ) |>

    dplyr::mutate(
      opleidingscode = factor(opleidingscode),
      locatie = factor(locatie),
      opleidingsvorm = factor(opleidingsvorm),
      postcode4_student_1okt = factor(postcode4_student_1okt),
      postcode4_vooropleiding_voorHO = factor(postcode4_vooropleiding_voorHO)
    ) |>

    ## Lange categorielabels inkorten en ongeldige postcodes verwijderen
    dplyr::mutate(
      opleidingsvorm = forcats::fct_recode(
        opleidingsvorm,
        "duaal" = "co\u00f6p-student of duaal onderwijs (vanaf het studiejaar 1998-1999)"
      ),
      sector = forcats::fct_recode(
        sector,
        "gedrag & maatschappij" = "gedrag en maatschappij",
        "taal & cultuur" = "taal en cultuur"
      ),
      ## 0010-0040 zijn onbekende postcodewaarden in de 1CHO-data
      postcode4_student_1okt = forcats::fct_recode(
        postcode4_student_1okt,
        NULL = "0010",
        NULL = "0020",
        NULL = "0030",
        NULL = "0040"
      ),
      postcode4_vooropleiding_voorHO = forcats::fct_recode(
        postcode4_vooropleiding_voorHO,
        NULL = "0010",
        NULL = "0020",
        NULL = "0030",
        NULL = "0040"
      ),
      opleidingsniveau = forcats::fct_recode(
        opleidingsniveau,
        NULL = "postinitiele master",
        bachelor = "ba",
        master = "ma"
      )
    ) |>

    ## Samengevatte indicatoren op basis van de gedetailleerde xjr-waarden
    dplyr::mutate(
      uitval = dplyr::case_when(
        uitval_xjr == 1 ~ "Uitgevallen binnen 1 jaar",
        uitval_xjr %in% 2:3 ~ "Uitgevallen in 2e of 3e jaar",
        uitval_xjr > 3 ~ "Uitgevallen na 3 jaar",
        TRUE ~ "Niet uitgevallen"
      ),
      studiewissel = dplyr::case_when(
        studiewissel_1jr ==
          "Gewisseld binnen 1 jaar" ~ "Gewisseld binnen 1 jaar",
        studiewissel_3jr ==
          "Gewisseld binnen 3 jaar" ~ "Gewisseld in het 2e of 3e jaar",
        TRUE ~ "Niet gewisseld"
      ),
      rendement = dplyr::case_when(
        rendement_5jr == "Diploma binnen 5 jaar" ~ "Diploma binnen 5 jaar",
        rendement_8jr == "Diploma binnen 8 jaar" ~ "Diploma binnen 5-8 jaar",
        rendement_8jr == "Diploma na 8 jaar" ~ "Diploma na 8 jaar",
        rendement_8jr == "Geen diploma" ~ "Geen diploma",
        rendement_8jr == "Onbekend (diplomajaar voor instroomjaar)" ~ "Onbekend"
      )
    )
}

## Bouwt een minimale set van vier indicatortibbles voor combineer_indicatoren
maak_input <- function(
  pgn = "A",
  inschrijvingsjaar = 2020,
  opleidingsvorm_label = "voltijd",
  croho_label = "techniek",
  postcode_student = "4818",
  postcode_vooropl = "4818",
  opleidingsniveau = "ba",
  status = "Zittend",
  uitval_xjr = NA_real_,
  uitval_1jr = "Na 1 jaar nog ingeschreven of diploma behaald",
  uitval_3jr = "Na 3 jaar nog ingeschreven of diploma behaald",
  studiewissel_1jr = "Niet gewisseld binnen 1 jaar",
  studiewissel_3jr = "Niet gewisseld binnen 3 jaar",
  rendement_3jr = "Geen diploma",
  rendement_5jr = "Geen diploma",
  rendement_8jr = "Geen diploma"
) {
  cohorten_instroom <- tibble(
    persoonsgebonden_nummer = pgn,
    inschrijvingsjaar = inschrijvingsjaar,
    eerstejaar_instelling = inschrijvingsjaar,
    geslacht_label = "vrouw",
    locatie_label = "Breda",
    opleiding_actueel_equivalent = "12345",
    opleidingsvorm_label = opleidingsvorm_label,
    type_hoger_onderwijs_binnen_soort_hoger_onderwijs = opleidingsniveau,
    indicatie_internationale_student_label = "geen internationale student",
    indicatie_eer_actueel_label = "eer",
    croho_onderdeel_actuele_opleiding_label = croho_label,
    leeftijd_per_peildatum_1_oktober = 19,
    postcodecijfers_student_op_1_oktober = postcode_student,
    postcodecijfers_van_de_hoogste_vooropl_voor_het_ho = postcode_vooropl,
    soort_diploma_instelling_label = NA_character_
  )

  rendement_indicatoren <- tibble(
    persoonsgebonden_nummer = pgn,
    eerstejaar_instelling = inschrijvingsjaar,
    jaar_eerste_diploma = NA_real_,
    verblijfsjaar_eerste_diploma = NA_integer_,
    diploma = NA_character_,
    rendement_xjaar = factor(NA_character_),
    rendement_3jr = factor(rendement_3jr),
    rendement_5jr = factor(rendement_5jr),
    rendement_8jr = factor(rendement_8jr)
  )

  uitval_indicatoren <- tibble(
    persoonsgebonden_nummer = pgn,
    laatste_jaar_inschrijving = NA_real_,
    diploma = NA_character_,
    status = factor(status),
    uitval_xjr = uitval_xjr,
    uitval_1jr = factor(uitval_1jr),
    uitval_3jr = factor(uitval_3jr)
  )

  studiewissel_indicatoren <- tibble(
    persoonsgebonden_nummer = pgn,
    studiewissel_1jr = factor(studiewissel_1jr),
    studiewissel_3jr = factor(studiewissel_3jr),
    opleidingscode_na_switch1jr = factor(NA_character_),
    opleidingsvorm_na_switch1jr = factor(NA_character_),
    opleidingsniveau_na_switch1jr = factor(NA_character_),
    sector_na_switch1jr = factor(NA_character_),
    opleidingscode_na_switch3jr = factor(NA_character_),
    opleidingsvorm_na_switch3jr = factor(NA_character_),
    opleidingsniveau_na_switch3jr = factor(NA_character_),
    sector_na_switch3jr = factor(NA_character_)
  )

  list(
    cohorten_instroom = cohorten_instroom,
    rendement_indicatoren = rendement_indicatoren,
    uitval_indicatoren = uitval_indicatoren,
    studiewissel_indicatoren = studiewissel_indicatoren
  )
}

roep_combineer <- function(inp) {
  ## suppressWarnings: fct_recode waarschuwt over ontbrekende niveaus in testdata
  ## die in productie wel aanwezig zijn omdat de factor alle mogelijke waarden bevat
  suppressWarnings(combineer_indicatoren(
    inp$cohorten_instroom,
    inp$rendement_indicatoren,
    inp$uitval_indicatoren,
    inp$studiewissel_indicatoren
  ))
}

## --- uitvoerstructuur ---

test_that("geeft één rij per student terug", {
  result <- roep_combineer(maak_input())
  expect_equal(nrow(result), 1)
})

test_that("bevat de verwachte kerncolomnamen", {
  result <- roep_combineer(maak_input())
  verwachte_kolommen <- c(
    "inschrijvingsjaar",
    "geslacht",
    "locatie",
    "opleidingscode",
    "opleidingsvorm",
    "opleidingsniveau",
    "status",
    "uitval",
    "studiewissel",
    "rendement"
  )
  expect_true(all(verwachte_kolommen %in% names(result)))
})

## --- locatie ---

test_that("neemt locatie_label direct over als locatie", {
  result <- roep_combineer(maak_input())
  expect_equal(as.character(result$locatie), "Breda")
})

## --- fct_recode: opleidingsvorm ---

test_that("hernoemt lange opleidingsvorm-label naar 'duaal'", {
  inp <- maak_input(
    opleidingsvorm_label = "coöp-student of duaal onderwijs (vanaf het studiejaar 1998-1999)"
  )
  result <- roep_combineer(inp)
  expect_equal(as.character(result$opleidingsvorm), "duaal")
})

test_that("laat andere opleidingsvorm-waarden ongewijzigd", {
  result <- roep_combineer(maak_input(opleidingsvorm_label = "voltijd"))
  expect_equal(as.character(result$opleidingsvorm), "voltijd")
})

## --- fct_recode: sector ---

test_that("hernoemt 'gedrag en maatschappij' naar 'gedrag & maatschappij'", {
  result <- roep_combineer(maak_input(croho_label = "gedrag en maatschappij"))
  expect_equal(as.character(result$sector), "gedrag & maatschappij")
})

test_that("hernoemt 'taal en cultuur' naar 'taal & cultuur'", {
  result <- roep_combineer(maak_input(croho_label = "taal en cultuur"))
  expect_equal(as.character(result$sector), "taal & cultuur")
})

test_that("laat andere sector-waarden ongewijzigd", {
  result <- roep_combineer(maak_input(croho_label = "techniek"))
  expect_equal(as.character(result$sector), "techniek")
})

## --- fct_recode: opleidingsniveau ---

test_that("hernoemt 'ba' naar 'bachelor'", {
  result <- roep_combineer(maak_input(opleidingsniveau = "ba"))
  expect_equal(as.character(result$opleidingsniveau), "bachelor")
})

test_that("hernoemt 'ma' naar 'master'", {
  result <- roep_combineer(maak_input(opleidingsniveau = "ma"))
  expect_equal(as.character(result$opleidingsniveau), "master")
})

test_that("verwijdert 'postinitiele master' uit opleidingsniveau", {
  result <- roep_combineer(maak_input(opleidingsniveau = "postinitiele master"))
  expect_true(is.na(result$opleidingsniveau))
})

## --- postcodefiltering ---

test_that("zet postcode 0010 om naar NA", {
  result <- roep_combineer(maak_input(postcode_student = "0010"))
  expect_true(is.na(result$postcode4_student_1okt))
})

test_that("zet alle vier ongeldige postcodes om naar NA", {
  for (postcode in c("0010", "0020", "0030", "0040")) {
    result <- roep_combineer(maak_input(postcode_student = postcode))
    expect_true(is.na(result$postcode4_student_1okt), label = postcode)
  }
})

test_that("laat geldige postcodes intact", {
  result <- roep_combineer(maak_input(postcode_student = "4818"))
  expect_equal(as.character(result$postcode4_student_1okt), "4818")
})

test_that("filtert ongeldige postcodes ook in vooropleidingspostcode", {
  result <- roep_combineer(maak_input(postcode_vooropl = "0030"))
  expect_true(is.na(result$postcode4_vooropleiding_voorHO))
})

## --- samengevatte indicatoren ---

test_that("bepaalt uitval correct op basis van uitval_xjr", {
  r1 <- roep_combineer(maak_input(uitval_xjr = 1, status = "Uitgevallen"))
  r2 <- roep_combineer(maak_input(uitval_xjr = 2, status = "Uitgevallen"))
  r3 <- roep_combineer(maak_input(uitval_xjr = 4, status = "Uitgevallen"))
  r4 <- roep_combineer(maak_input())

  expect_equal(r1$uitval, "Uitgevallen binnen 1 jaar")
  expect_equal(r2$uitval, "Uitgevallen in 2e of 3e jaar")
  expect_equal(r3$uitval, "Uitgevallen na 3 jaar")
  expect_equal(r4$uitval, "Niet uitgevallen")
})

test_that("bepaalt studiewissel correct op basis van studiewissel_1jr en _3jr", {
  r1 <- roep_combineer(maak_input(studiewissel_1jr = "Gewisseld binnen 1 jaar"))
  r2 <- roep_combineer(maak_input(studiewissel_3jr = "Gewisseld binnen 3 jaar"))
  r3 <- roep_combineer(maak_input())

  expect_equal(r1$studiewissel, "Gewisseld binnen 1 jaar")
  expect_equal(r2$studiewissel, "Gewisseld in het 2e of 3e jaar")
  expect_equal(r3$studiewissel, "Niet gewisseld")
})

test_that("geeft 'Onbekend' als rendement_8jr de onbekend-waarde heeft", {
  result <- roep_combineer(maak_input(
    rendement_5jr = "Onbekend (diplomajaar voor instroomjaar)",
    rendement_8jr = "Onbekend (diplomajaar voor instroomjaar)"
  ))
  expect_equal(result$rendement, "Onbekend")
})

test_that("bepaalt rendement correct op basis van rendement_5jr en _8jr", {
  r1 <- roep_combineer(maak_input(
    rendement_5jr = "Diploma binnen 5 jaar",
    rendement_8jr = "Diploma binnen 8 jaar"
  ))
  r2 <- roep_combineer(maak_input(
    rendement_5jr = "Diploma na 5 jaar",
    rendement_8jr = "Diploma binnen 8 jaar"
  ))
  r3 <- roep_combineer(maak_input(
    rendement_5jr = "Diploma na 5 jaar",
    rendement_8jr = "Diploma na 8 jaar"
  ))
  r4 <- roep_combineer(maak_input())

  expect_equal(r1$rendement, "Diploma binnen 5 jaar")
  expect_equal(r2$rendement, "Diploma binnen 5-8 jaar")
  expect_equal(r3$rendement, "Diploma na 8 jaar")
  expect_equal(r4$rendement, "Geen diploma")
})

#' Bereken studiewisselindicatoren per cohort
#'
#' Bepaalt per student of zij binnen 1 of 3 jaar van opleiding zijn gewisseld.
#' Studenten die al zijn uitgevallen of gediplomeerd in de meetperiode worden
#' buiten beschouwing gelaten.
#'
#' @param basisbestand Tibble zoals gemaakt door [maak_basisbestand()]
#' @param cohorten_instroom Tibble zoals gemaakt door [maak_instroom_cohort()]
#' @param diploma_behaald Tibble zoals gemaakt door [maak_diploma_behaald()]
#' @param uitval_indicatoren Tibble zoals gemaakt door [bereken_uitval()]
#'
#' @return Een tibble met kolommen `persoonsgebonden_nummer`,
#'   `studiewissel_1jr`, `studiewissel_3jr` (factoren) en aanvullende
#'   switch-kolommen met de opleiding, opleidingsvorm, niveau en sector na
#'   de wissel
#'
#' @examples
#' basis <- tibble::tibble(
#'   persoonsgebonden_nummer = c("S1", "S1"),
#'   verblijfsjaar_actuele_instelling = c(1L, 2L),
#'   soort_inschrijving_actuele_instelling_label =
#'     "hoofdinschrijving binnen het domein actuele instelling",
#'   opleiding_actueel_equivalent = "34401",
#'   inschrijvingsjaar = c(2020L, 2021L),
#'   opleidingsvorm = "voltijd",
#'   type_hoger_onderwijs_binnen_soort_hoger_onderwijs = "bachelor",
#'   croho_onderdeel_actuele_opleiding = "techniek"
#' )
#' cohort <- tibble::tibble(
#'   persoonsgebonden_nummer = "S1",
#'   inschrijvingsjaar = 2020L,
#'   eerstejaar_instelling = 2020L,
#'   soort_diploma_instelling_label = NA_character_,
#'   opleidingsvorm_label = "voltijd"
#' )
#' diploma <- tibble::tibble(
#'   persoonsgebonden_nummer = character(0),
#'   jaar_eerste_diploma = integer(0),
#'   verblijfsjaar_eerste_diploma = integer(0),
#'   diploma = character(0)
#' )
#' uitval <- tibble::tibble(
#'   persoonsgebonden_nummer = "S1",
#'   uitval_xjr = NA_real_
#' )
#' bereken_studiewissel(basis, cohort, diploma, uitval)
#' @export
bereken_studiewissel <- function(
  basisbestand,
  cohorten_instroom,
  diploma_behaald,
  uitval_indicatoren
) {
  ## Uitval binnen 1 of 3 jaar per student bepalen voor exclusie
  uitval_1_3_jaar <- uitval_indicatoren |>
    dplyr::select(persoonsgebonden_nummer, uitval_xjr) |>
    dplyr::mutate(
      uitval_een_drie = factor(dplyr::case_when(
        uitval_xjr == 1 ~ "binnen 1 jaar",
        uitval_xjr > 1 & uitval_xjr <= 3 ~ "binnen 2- en 3de jaar",
        TRUE ~ "niet uitgevallen eerste drie jaar"
      ))
    )

  ## Koppel diploma en uitvalinfo aan cohort voor zittend-bepaling
  zittend_per_cohort <- cohorten_instroom |>
    dplyr::select(-soort_diploma_instelling_label) |>
    dplyr::left_join(diploma_behaald, by = "persoonsgebonden_nummer") |>
    dplyr::select(
      persoonsgebonden_nummer,
      inschrijvingsjaar,
      eerstejaar_instelling,
      jaar_eerste_diploma,
      opleidingsvorm_label
    ) |>
    dplyr::mutate(
      diploma_na_x_jaar = jaar_eerste_diploma - eerstejaar_instelling + 1,
      diploma_een_drie = factor(dplyr::case_when(
        diploma_na_x_jaar <= 1 ~ "binnen 1 jaar of daarvoor",
        diploma_na_x_jaar > 1 &
          diploma_na_x_jaar <= 3 ~ "binnen 2- en 3de jaar",
        TRUE ~ "geen diploma in eerste drie jaar"
      ))
    ) |>
    dplyr::left_join(uitval_1_3_jaar, by = "persoonsgebonden_nummer")

  ## Bepaal de populatie die een wissel kon maken in het 1e resp. 3e jaar
  zittend_1jr <- zittend_per_cohort |>
    dplyr::filter(diploma_een_drie != "binnen 1 jaar of daarvoor") |>
    dplyr::filter(uitval_een_drie != "binnen 1 jaar")

  zittend_3jr <- zittend_per_cohort |>
    dplyr::filter(diploma_een_drie != "binnen 1 jaar of daarvoor") |>
    dplyr::filter(diploma_een_drie != "binnen 2- en 3de jaar") |>
    dplyr::filter(uitval_een_drie != "binnen 1 jaar") |>
    dplyr::filter(uitval_een_drie != "binnen 2- en 3de jaar")

  wissel_1jr <- bereken_wissel_xjr(
    basisbestand = basisbestand,
    zittend = zittend_1jr,
    verblijfsjaren = c(1, 2),
    doeljaar = 2,
    label_gewisseld = "Gewisseld binnen 1 jaar",
    label_niet = "Niet gewisseld binnen 1 jaar",
    suffix = "1jr"
  )

  wissel_3jr <- bereken_wissel_xjr(
    basisbestand = basisbestand,
    zittend = zittend_3jr,
    verblijfsjaren = c(1, 4),
    doeljaar = 4,
    label_gewisseld = "Gewisseld binnen 3 jaar",
    label_niet = "Niet gewisseld binnen 3 jaar",
    suffix = "3jr"
  )

  cohorten_instroom |>
    dplyr::left_join(wissel_1jr, by = "persoonsgebonden_nummer") |>
    dplyr::left_join(wissel_3jr, by = "persoonsgebonden_nummer") |>
    dplyr::mutate(
      studiewissel_1jr = dplyr::case_when(
        !(persoonsgebonden_nummer %in%
          zittend_1jr$persoonsgebonden_nummer) ~ "Geen switch bepaald",
        TRUE ~ studiewissel_1jr
      ),
      studiewissel_3jr = dplyr::case_when(
        !(persoonsgebonden_nummer %in%
          zittend_3jr$persoonsgebonden_nummer) ~ "Geen switch bepaald",
        TRUE ~ studiewissel_3jr
      )
    ) |>
    dplyr::mutate(
      studiewissel_1jr = forcats::fct_na_value_to_level(
        studiewissel_1jr,
        "Niet gewisseld binnen 1 jaar"
      ),
      studiewissel_3jr = forcats::fct_na_value_to_level(
        studiewissel_3jr,
        "Niet gewisseld binnen 3 jaar"
      )
    ) |>
    dplyr::select(
      persoonsgebonden_nummer,
      dplyr::starts_with("studiewissel"),
      dplyr::contains("switch")
    )
}

#' Bereken studiewissel voor een specifiek verblijfsjaar-paar
#'
#' Hulpfunctie die de switchlogica hergebruikt voor zowel de 1jr- als
#' 3jr-meting. Een wissel wordt vastgesteld als een student in jaar `doeljaar`
#' een andere opleidingscode heeft dan in jaar 1, en het verschil in
#' kalenderjaren gelijk is aan `doeljaar - 1`.
#'
#' @param basisbestand Tibble zoals gemaakt door [maak_basisbestand()]
#' @param zittend Tibble met de populatie waarvoor de wissel bepaald wordt
#' @param verblijfsjaren Integer vector van lengte 2: begin- en eindjaar van
#'   het meetvenster, bijv. `c(1, 2)` of `c(1, 4)`
#' @param doeljaar Integer, het verblijfsjaar waarop de wissel wordt gemeten
#' @param label_gewisseld Character, label als student gewisseld is
#' @param label_niet Character, label als student niet gewisseld is
#' @param suffix Character, achtervoegsel voor de kolomnamen (`"1jr"` of
#'   `"3jr"`)
#'
#' @return Een tibble met alleen de studenten die gewisseld zijn, met kolommen
#'   voor wissel, nieuwe opleidingscode, opleidingsvorm, niveau en sector
bereken_wissel_xjr <- function(
  basisbestand,
  zittend,
  verblijfsjaren,
  doeljaar,
  label_gewisseld,
  label_niet,
  suffix
) {
  bewerkt <- basisbestand |>
    dplyr::filter(
      persoonsgebonden_nummer %in% zittend$persoonsgebonden_nummer
    ) |>
    dplyr::filter(
      verblijfsjaar_actuele_instelling %in% verblijfsjaren,
      soort_inschrijving_actuele_instelling_label ==
        "hoofdinschrijving binnen het domein actuele instelling"
    )

  freq <- bewerkt |> dplyr::count(persoonsgebonden_nummer)

  if (any(freq$n > 2)) {
    rlang::abort(
      "Meer dan twee regels per student in het switchcohortbestand gevonden!"
    )
  }

  persnr_2jr <- freq |> dplyr::filter(.data$n == 2)

  dubbel <- bewerkt |>
    dplyr::filter(
      persoonsgebonden_nummer %in% persnr_2jr$persoonsgebonden_nummer
    )

  if (
    sum(dubbel$verblijfsjaar_actuele_instelling == verblijfsjaren[1]) !=
      sum(dubbel$verblijfsjaar_actuele_instelling == verblijfsjaren[2])
  ) {
    rlang::abort(paste(
      "Aantal verblijfsjaren",
      verblijfsjaren[1],
      "en",
      verblijfsjaren[2],
      "is niet gelijk"
    ))
  }

  wissel_col <- paste0("studiewissel_", suffix)
  opl_col <- paste0("opleidingscode_na_switch", suffix)
  vorm_col <- paste0("opleidingsvorm_na_switch", suffix)
  niveau_col <- paste0("opleidingsniveau_na_switch", suffix)
  sector_col <- paste0("sector_na_switch", suffix)

  dubbel |>
    dplyr::arrange(persoonsgebonden_nummer, verblijfsjaar_actuele_instelling) |>
    dplyr::mutate(
      persnr_dubbel = dplyr::if_else(
        persoonsgebonden_nummer == dplyr::lag(persoonsgebonden_nummer),
        "zelfde student",
        "andere student"
      ),
      opl_dubbel = dplyr::if_else(
        opleiding_actueel_equivalent ==
          dplyr::lag(opleiding_actueel_equivalent),
        "zelfde opleiding",
        "andere opleiding"
      ),
      !!wissel_col := dplyr::if_else(
        verblijfsjaar_actuele_instelling == doeljaar &
          persnr_dubbel == "zelfde student" &
          opl_dubbel == "andere opleiding",
        label_gewisseld,
        label_niet
      ),
      verschil_kalenderjaren = inschrijvingsjaar -
        dplyr::lag(inschrijvingsjaar),
      !!opl_col := dplyr::if_else(
        .data[[wissel_col]] == label_gewisseld,
        as.character(opleiding_actueel_equivalent),
        NA_character_
      ),
      !!vorm_col := dplyr::if_else(
        .data[[wissel_col]] == label_gewisseld,
        as.character(opleidingsvorm),
        NA_character_
      ),
      !!niveau_col := dplyr::if_else(
        .data[[wissel_col]] == label_gewisseld,
        as.character(type_hoger_onderwijs_binnen_soort_hoger_onderwijs),
        NA_character_
      ),
      !!sector_col := dplyr::if_else(
        .data[[wissel_col]] == label_gewisseld,
        as.character(croho_onderdeel_actuele_opleiding),
        NA_character_
      )
    ) |>
    dplyr::mutate(dplyr::across(
      c(!!wissel_col, !!opl_col, !!vorm_col, !!niveau_col, !!sector_col),
      factor
    )) |>
    dplyr::select(
      persoonsgebonden_nummer,
      verschil_kalenderjaren,
      !!wissel_col,
      !!opl_col,
      !!vorm_col,
      !!niveau_col,
      !!sector_col
    ) |>
    dplyr::filter(
      .data[[wissel_col]] == label_gewisseld,
      verschil_kalenderjaren == (doeljaar - 1)
    )
}

maak_wissel_rijen <- function(
  pgn,
  verblijfsjaren,
  opleidingen,
  inschrijvingsjaren
) {
  tibble(
    persoonsgebonden_nummer = pgn,
    verblijfsjaar_actuele_instelling = as.integer(verblijfsjaren),
    soort_inschrijving_actuele_instelling_label = HOOFD_INSCHRIJVING,
    opleiding_actueel_equivalent = opleidingen,
    inschrijvingsjaar = inschrijvingsjaren,
    opleidingsvorm = "voltijd",
    type_hoger_onderwijs_binnen_soort_hoger_onderwijs = "bachelor",
    croho_onderdeel_actuele_opleiding = "techniek"
  )
}

## --- bereken_wissel_xjr: wissels ---

test_that("detecteert opleidingswissel correct bij 1jr", {
  basisbestand <- bind_rows(
    maak_wissel_rijen(c("A", "A"), c(1, 2), c("opl1", "opl2"), c(2019, 2020)),
    maak_wissel_rijen(c("B", "B"), c(1, 2), c("opl1", "opl1"), c(2019, 2020))
  )
  zittend <- tibble(persoonsgebonden_nummer = c("A", "B"))

  result <- bereken_wissel_xjr(
    basisbestand,
    zittend,
    verblijfsjaren = c(1, 2),
    doeljaar = 2,
    label_gewisseld = "Gewisseld binnen 1 jaar",
    label_niet = "Niet gewisseld binnen 1 jaar",
    suffix = "1jr"
  )

  expect_equal(nrow(result), 1)
  expect_equal(result$persoonsgebonden_nummer, "A")
  expect_equal(as.character(result$studiewissel_1jr), "Gewisseld binnen 1 jaar")
})

test_that("detecteert opleidingswissel correct bij 3jr", {
  basisbestand <- bind_rows(
    maak_wissel_rijen(c("A", "A"), c(1, 4), c("opl1", "opl2"), c(2015, 2018)),
    maak_wissel_rijen(c("B", "B"), c(1, 4), c("opl1", "opl1"), c(2015, 2018))
  )
  zittend <- tibble(persoonsgebonden_nummer = c("A", "B"))

  result <- bereken_wissel_xjr(
    basisbestand,
    zittend,
    verblijfsjaren = c(1, 4),
    doeljaar = 4,
    label_gewisseld = "Gewisseld binnen 3 jaar",
    label_niet = "Niet gewisseld binnen 3 jaar",
    suffix = "3jr"
  )

  expect_equal(nrow(result), 1)
  expect_equal(result$persoonsgebonden_nummer, "A")
  expect_equal(as.character(result$studiewissel_3jr), "Gewisseld binnen 3 jaar")
})

test_that("geeft lege tibble terug als niemand gewisseld is", {
  basisbestand <- bind_rows(
    maak_wissel_rijen(c("A", "A"), c(1, 2), c("opl1", "opl1"), c(2019, 2020)),
    maak_wissel_rijen(c("B", "B"), c(1, 2), c("opl2", "opl2"), c(2019, 2020))
  )
  zittend <- tibble(persoonsgebonden_nummer = c("A", "B"))

  result <- bereken_wissel_xjr(
    basisbestand,
    zittend,
    verblijfsjaren = c(1, 2),
    doeljaar = 2,
    label_gewisseld = "Gewisseld binnen 1 jaar",
    label_niet = "Niet gewisseld binnen 1 jaar",
    suffix = "1jr"
  )

  expect_equal(nrow(result), 0)
})

## --- bereken_wissel_xjr: populatiefilter ---

test_that("negeert studenten die niet in zittend zitten", {
  basisbestand <- bind_rows(
    maak_wissel_rijen(c("A", "A"), c(1, 2), c("opl1", "opl2"), c(2019, 2020)),
    maak_wissel_rijen(c("B", "B"), c(1, 2), c("opl1", "opl2"), c(2019, 2020))
  )
  zittend <- tibble(persoonsgebonden_nummer = "A")

  result <- bereken_wissel_xjr(
    basisbestand,
    zittend,
    verblijfsjaren = c(1, 2),
    doeljaar = 2,
    label_gewisseld = "Gewisseld binnen 1 jaar",
    label_niet = "Niet gewisseld binnen 1 jaar",
    suffix = "1jr"
  )

  expect_equal(nrow(result), 1)
  expect_equal(result$persoonsgebonden_nummer, "A")
})

test_that("geeft lege tibble als zittend leeg is", {
  basisbestand <- maak_wissel_rijen(
    c("A", "A"),
    c(1, 2),
    c("opl1", "opl2"),
    c(2019, 2020)
  )
  zittend <- tibble(persoonsgebonden_nummer = character(0))

  result <- bereken_wissel_xjr(
    basisbestand,
    zittend,
    verblijfsjaren = c(1, 2),
    doeljaar = 2,
    label_gewisseld = "Gewisseld binnen 1 jaar",
    label_niet = "Niet gewisseld binnen 1 jaar",
    suffix = "1jr"
  )

  expect_equal(nrow(result), 0)
})

## --- bereken_wissel_xjr: kalenderjaarfilter ---

test_that("sluit studenten uit met verkeerd verschil in kalenderjaren", {
  ## Verschil is 2 jaar terwijl doeljaar - 1 = 1 verwacht
  basisbestand <- maak_wissel_rijen(
    c("A", "A"),
    c(1, 2),
    c("opl1", "opl2"),
    c(2018, 2020)
  )
  zittend <- tibble(persoonsgebonden_nummer = "A")

  result <- bereken_wissel_xjr(
    basisbestand,
    zittend,
    verblijfsjaren = c(1, 2),
    doeljaar = 2,
    label_gewisseld = "Gewisseld binnen 1 jaar",
    label_niet = "Niet gewisseld binnen 1 jaar",
    suffix = "1jr"
  )

  expect_equal(nrow(result), 0)
})

## --- bereken_wissel_xjr: uitvoerkolommen ---

test_that("bevat de juiste switchkolommen in de output", {
  basisbestand <- maak_wissel_rijen(
    c("A", "A"),
    c(1, 2),
    c("opl1", "opl2"),
    c(2019, 2020)
  )
  zittend <- tibble(persoonsgebonden_nummer = "A")

  result <- bereken_wissel_xjr(
    basisbestand,
    zittend,
    verblijfsjaren = c(1, 2),
    doeljaar = 2,
    label_gewisseld = "Gewisseld binnen 1 jaar",
    label_niet = "Niet gewisseld binnen 1 jaar",
    suffix = "1jr"
  )

  expect_true("studiewissel_1jr" %in% names(result))
  expect_true("opleidingscode_na_switch1jr" %in% names(result))
  expect_true("opleidingsvorm_na_switch1jr" %in% names(result))
  expect_true("opleidingsniveau_na_switch1jr" %in% names(result))
  expect_true("sector_na_switch1jr" %in% names(result))
})

test_that("vult opleidingscode_na_switch in bij wissel", {
  basisbestand <- maak_wissel_rijen(
    c("A", "A"),
    c(1, 2),
    c("opl1", "opl2"),
    c(2019, 2020)
  )
  zittend <- tibble(persoonsgebonden_nummer = "A")

  result <- bereken_wissel_xjr(
    basisbestand,
    zittend,
    verblijfsjaren = c(1, 2),
    doeljaar = 2,
    label_gewisseld = "Gewisseld binnen 1 jaar",
    label_niet = "Niet gewisseld binnen 1 jaar",
    suffix = "1jr"
  )

  expect_equal(as.character(result$opleidingscode_na_switch1jr), "opl2")
})

## --- bereken_wissel_xjr: foutcondities ---

test_that("geeft fout bij meer dan 2 rijen per student", {
  basisbestand <- maak_wissel_rijen(
    c("A", "A", "A"),
    c(1, 1, 2),
    c("opl1", "opl1", "opl2"),
    c(2019, 2019, 2020)
  )
  zittend <- tibble(persoonsgebonden_nummer = "A")

  expect_error(bereken_wissel_xjr(
    basisbestand,
    zittend,
    verblijfsjaren = c(1, 2),
    doeljaar = 2,
    label_gewisseld = "Gewisseld binnen 1 jaar",
    label_niet = "Niet gewisseld binnen 1 jaar",
    suffix = "1jr"
  ))
})

test_that("geeft fout bij ongelijk aantal verblijfsjaar 1 en 2 rijen", {
  ## Student A heeft twee rijen met verblijfsjaar 1 en geen met verblijfsjaar 2
  basisbestand <- maak_wissel_rijen(
    c("A", "A"),
    c(1, 1),
    c("opl1", "opl2"),
    c(2019, 2019)
  )
  zittend <- tibble(persoonsgebonden_nummer = "A")

  expect_error(bereken_wissel_xjr(
    basisbestand,
    zittend,
    verblijfsjaren = c(1, 2),
    doeljaar = 2,
    label_gewisseld = "Gewisseld binnen 1 jaar",
    label_niet = "Niet gewisseld binnen 1 jaar",
    suffix = "1jr"
  ))
})

#' Start het Staat van Onderwijsinstelling dashboard
#'
#' Opent de interactieve Shiny app waarmee je een 1CHO CSV-bestand kunt
#' uploaden en studie-indicatoren kunt verkennen.
#'
#' @return No return value, called for side effects.
#' @examples
#' if (interactive()) {
#'   start_dashboard()
#' }
#' @export
start_dashboard <- function() {
  app_pad <- system.file("app", package = "staat1cho")
  if (!nzchar(app_pad)) {
    cli::cli_abort(
      "De app kon niet gevonden worden. Zorg dat staat1cho correct is geinstalleerd."
    )
  }
  shiny::runApp(app_pad)
}

#' Lees het 1CHO-bestand in en voeg label-kolommen toe
#'
#' Leest een semicolongescheiden CSV-bestand (UTF-8) in en voegt
#' extra `_label`-kolommen toe voor gebruik in rapportages. Het pakket
#' bevat een klein synthetisch voorbeeldbestand zonder echte persoonsgegevens
#' (`inst/extdata/voorbeeld_1cho.csv`).
#'
#' @param pad_invoer Pad naar het semicolongescheiden CSV-bestand (UTF-8)
#'
#' @return Een tibble met alle 1CHO-regels plus extra `_label`-kolommen die de
#'   originele categorische waarden bewaren voor gebruik in rapportages
#'
#' @examples
#' # voorbeeld_1cho.csv is a small synthetic dataset bundled with the package
#' pad <- system.file("extdata/voorbeeld_1cho.csv", package = "staat1cho")
#' basis <- suppressMessages(maak_basisbestand(pad))
#' @export
maak_basisbestand <- function(pad_invoer) {
  invoer <- readr::read_csv2(
    pad_invoer,
    locale = readr::locale(encoding = "UTF-8")
  )

  if (!"vestigingsnummer_gemeentenaam_volgens_rio" %in% names(invoer)) {
    cli::cli_abort(
      "Kolom {.val vestigingsnummer_gemeentenaam_volgens_rio} niet gevonden in het invoerbestand."
    )
  }

  invoer |>
    dplyr::mutate(
      verblijfsjaar_actuele_instelling = as.integer(
        verblijfsjaar_actuele_instelling
      ),
      diplomajaar = suppressWarnings(as.integer(diplomajaar)),
      soort_inschrijving_actuele_instelling_label = soort_inschrijving_actuele_instelling,
      geslacht_label = geslacht,
      opleidingsvorm_label = opleidingsvorm,
      indicatie_internationale_student_label = indicatie_internationale_student,
      indicatie_eer_actueel_label = indicatie_eer_actueel,
      croho_onderdeel_actuele_opleiding_label = croho_onderdeel_actuele_opleiding,
      soort_diploma_instelling_label = soort_diploma_instelling,
      locatie_label = vestigingsnummer_gemeentenaam_volgens_rio
    )
}

#' Maak een eerstejaarscohort per instelling
#'
#' Filtert het basisbestand op het opgegeven soort hoger onderwijs,
#' hoofdinschrijvingen en eerste verblijfsjaar.
#'
#' @param basisbestand Tibble zoals gemaakt door [maak_basisbestand()]
#' @param soort_ho Character vector met toegestane waarden van
#'   `soort_hoger_onderwijs`, bijv. `c("hoger beroepsonderwijs", "hbo")`
#'
#' @return Een tibble met één rij per student, aangevuld met kolom
#'   `eerstejaar_instelling` (= inschrijvingsjaar). Gooit een fout als er
#'   dubbele persoonsgebonden nummers zijn.
#'
#' @examples
#' basis <- tibble::tibble(
#'   persoonsgebonden_nummer = c("S1", "S2", "S3"),
#'   soort_hoger_onderwijs = c("hbo", "wo", "hbo"),
#'   soort_inschrijving_actuele_instelling_label =
#'     "hoofdinschrijving binnen het domein actuele instelling",
#'   verblijfsjaar_actuele_instelling = 1L,
#'   inschrijvingsjaar = 2020L,
#'   soort_diploma_instelling_label = NA_character_
#' )
#' maak_instroom_cohort(basis, "hbo")
#' @export
maak_instroom_cohort <- function(basisbestand, soort_ho) {
  cohort <- basisbestand |>
    dplyr::filter(soort_hoger_onderwijs %in% soort_ho) |>
    dplyr::filter(
      soort_inschrijving_actuele_instelling_label ==
        "hoofdinschrijving binnen het domein actuele instelling",
      verblijfsjaar_actuele_instelling == 1
    ) |>
    dplyr::mutate(eerstejaar_instelling = inschrijvingsjaar)

  if (any(duplicated(cohort$persoonsgebonden_nummer))) {
    rlang::abort(
      "Dubbele studentnummers in het instroomcohortbestand gevonden!"
    )
  }

  cohort
}

## --- maak_instroom_cohort ---

test_that("filtert op soort_ho", {
  basisbestand <- tibble(
    persoonsgebonden_nummer = c("A", "B", "C"),
    soort_hoger_onderwijs = c("hbo", "wo", "hbo"),
    soort_inschrijving_actuele_instelling_label = HOOFD_INSCHRIJVING,
    verblijfsjaar_actuele_instelling = 1L,
    inschrijvingsjaar = 2020
  )

  result <- maak_instroom_cohort(basisbestand, "hbo")

  expect_setequal(result$persoonsgebonden_nummer, c("A", "C"))
})

test_that("accepteert meerdere soort_ho-waarden", {
  basisbestand <- tibble(
    persoonsgebonden_nummer = c("A", "B", "C"),
    soort_hoger_onderwijs = c("hoger beroepsonderwijs", "hbo", "wo"),
    soort_inschrijving_actuele_instelling_label = HOOFD_INSCHRIJVING,
    verblijfsjaar_actuele_instelling = 1L,
    inschrijvingsjaar = 2020
  )

  result <- maak_instroom_cohort(
    basisbestand,
    c("hoger beroepsonderwijs", "hbo")
  )

  expect_setequal(result$persoonsgebonden_nummer, c("A", "B"))
})

test_that("behoudt alleen verblijfsjaar 1", {
  basisbestand <- tibble(
    persoonsgebonden_nummer = c("A", "A", "A"),
    soort_hoger_onderwijs = "hbo",
    soort_inschrijving_actuele_instelling_label = HOOFD_INSCHRIJVING,
    verblijfsjaar_actuele_instelling = c(1L, 2L, 3L),
    inschrijvingsjaar = c(2020, 2021, 2022)
  )

  result <- maak_instroom_cohort(basisbestand, "hbo")

  expect_equal(nrow(result), 1)
  expect_equal(result$verblijfsjaar_actuele_instelling, 1L)
})

test_that("filtert niet-hoofdinschrijvingen eruit", {
  basisbestand <- tibble(
    persoonsgebonden_nummer = c("A", "B", "C"),
    soort_hoger_onderwijs = "hbo",
    soort_inschrijving_actuele_instelling_label = c(
      HOOFD_INSCHRIJVING,
      "neveninschrijving binnen het domein actuele instelling",
      "neveninschrijving binnen het domein hoger onderwijs"
    ),
    verblijfsjaar_actuele_instelling = 1L,
    inschrijvingsjaar = 2020
  )

  result <- maak_instroom_cohort(basisbestand, "hbo")

  expect_equal(nrow(result), 1)
  expect_equal(result$persoonsgebonden_nummer, "A")
})

test_that("zet eerstejaar_instelling gelijk aan inschrijvingsjaar", {
  basisbestand <- tibble(
    persoonsgebonden_nummer = "A",
    soort_hoger_onderwijs = "hbo",
    soort_inschrijving_actuele_instelling_label = HOOFD_INSCHRIJVING,
    verblijfsjaar_actuele_instelling = 1L,
    inschrijvingsjaar = 2021
  )

  result <- maak_instroom_cohort(basisbestand, "hbo")

  expect_equal(result$eerstejaar_instelling, 2021)
})

test_that("geeft fout bij dubbele persoonsgebonden_nummers", {
  basisbestand <- tibble(
    persoonsgebonden_nummer = c("A", "A"),
    soort_hoger_onderwijs = "hbo",
    soort_inschrijving_actuele_instelling_label = HOOFD_INSCHRIJVING,
    verblijfsjaar_actuele_instelling = 1L,
    inschrijvingsjaar = 2020
  )

  expect_error(maak_instroom_cohort(basisbestand, "hbo"))
})

test_that("geeft lege tibble terug bij geen matches", {
  basisbestand <- tibble(
    persoonsgebonden_nummer = "A",
    soort_hoger_onderwijs = "wo",
    soort_inschrijving_actuele_instelling_label = HOOFD_INSCHRIJVING,
    verblijfsjaar_actuele_instelling = 1L,
    inschrijvingsjaar = 2020
  )

  result <- maak_instroom_cohort(basisbestand, "hbo")

  expect_equal(nrow(result), 0)
})

test_that("behoudt alle originele kolommen in de output", {
  basisbestand <- tibble(
    persoonsgebonden_nummer = "A",
    soort_hoger_onderwijs = "hbo",
    soort_inschrijving_actuele_instelling_label = HOOFD_INSCHRIJVING,
    verblijfsjaar_actuele_instelling = 1L,
    inschrijvingsjaar = 2020,
    extra_kolom = "x"
  )

  result <- maak_instroom_cohort(basisbestand, "hbo")

  expect_true("extra_kolom" %in% names(result))
  expect_true("eerstejaar_instelling" %in% names(result))
})

test_that("past tegelijk alle drie filters toe", {
  ## Student B valt af op soort_ho, C op verblijfsjaar, D op inschrijvingstype
  basisbestand <- tibble(
    persoonsgebonden_nummer = c("A", "B", "C", "D"),
    soort_hoger_onderwijs = c("hbo", "wo", "hbo", "hbo"),
    soort_inschrijving_actuele_instelling_label = c(
      HOOFD_INSCHRIJVING,
      HOOFD_INSCHRIJVING,
      HOOFD_INSCHRIJVING,
      "neveninschrijving"
    ),
    verblijfsjaar_actuele_instelling = c(1L, 1L, 2L, 1L),
    inschrijvingsjaar = c(2020, 2020, 2019, 2020)
  )

  result <- maak_instroom_cohort(basisbestand, "hbo")

  expect_equal(nrow(result), 1)
  expect_equal(result$persoonsgebonden_nummer, "A")
})

#' Maak een bestand met het vroegst behaalde diploma per student
#'
#' Filtert het basisbestand op diplomasoorten die gelden als afgeronde opleiding
#' en behoudt per student alleen het eerste diploma op basis van diplomajaar.
#'
#' @param basisbestand Tibble zoals gemaakt door [maak_basisbestand()]
#'
#' @return Een tibble met één rij per student met kolommen
#'   `persoonsgebonden_nummer`, `jaar_eerste_diploma`,
#'   `verblijfsjaar_eerste_diploma` en `diploma`. Gooit een fout bij
#'   dubbele persoonsgebonden nummers.
#'
#' @examples
#' basis <- tibble::tibble(
#'   persoonsgebonden_nummer = c("S1", "S2"),
#'   soort_diploma_instelling = c(
#'     "Hoofd-bachelor-diploma binnen de actuele instelling",
#'     NA_character_
#'   ),
#'   diplomajaar = c(2022L, NA_integer_),
#'   verblijfsjaar_actuele_instelling = c(3L, 1L)
#' )
#' maak_diploma_behaald(basis)
#' @export
maak_diploma_behaald <- function(basisbestand) {
  diplomas <- c(
    "Hoofd-bachelor-diploma binnen de actuele instelling",
    "Neven-bachelor-diploma binnen de actuele instelling",
    "Hoofd-master-diploma binnen de actuele instelling",
    "Neven-master-diploma binnen de actuele instelling",
    "Hoofd-doctoraal-diploma binnen de actuele instelling",
    "Neven-doctoraal-diploma binnen de actuele instelling",
    "Hoofddiploma beroepsfase/voortgezet binnen de actuele instelling",
    "Nevendiploma beroepsfase/voortgezet binnen de actuele instelling",
    "Hoofddiploma associate degree binnen de actuele instelling",
    "Nevendiploma associate degree binnen de actuele instelling",
    "Hoofddiploma postinitiele master binnen de actuele instelling",
    "Nevendiploma postinitiele master binnen de actuele instelling"
  )

  diploma_behaald <- basisbestand |>
    dplyr::filter(soort_diploma_instelling %in% diplomas) |>
    ## diplomajaar == 0 betekent geen jaar geregistreerd in de 1CHO-data
    dplyr::mutate(diplomajaar = dplyr::na_if(diplomajaar, 0)) |>
    dplyr::group_by(persoonsgebonden_nummer) |>
    dplyr::arrange(diplomajaar, .by_group = TRUE) |>
    dplyr::distinct(persoonsgebonden_nummer, .keep_all = TRUE) |>
    dplyr::ungroup() |>
    dplyr::mutate(
      jaar_eerste_diploma = diplomajaar,
      verblijfsjaar_eerste_diploma = verblijfsjaar_actuele_instelling,
      diploma = "Diploma behaald (excl. propedeuse)"
    ) |>
    dplyr::select(
      persoonsgebonden_nummer,
      jaar_eerste_diploma,
      verblijfsjaar_eerste_diploma,
      diploma
    )

  if (any(duplicated(diploma_behaald$persoonsgebonden_nummer))) {
    rlang::abort("Dubbele studentnummers in het diplomabestand gevonden!")
  }

  diploma_behaald
}

#' Bereken rendementsindicatoren per cohort
#'
#' Koppelt diplomagegevens aan het instroomcohort en berekent of een student
#' binnen 3, 5 of 8 jaar een diploma heeft behaald.
#'
#' @param cohorten_instroom Tibble zoals gemaakt door [maak_instroom_cohort()]
#' @param diploma_behaald Tibble zoals gemaakt door [maak_diploma_behaald()]
#'
#' @return Een tibble met kolommen `persoonsgebonden_nummer`,
#'   `eerstejaar_instelling`, `jaar_eerste_diploma`,
#'   `verblijfsjaar_eerste_diploma`, `diploma`, `rendement_xjaar`,
#'   en factorkolommen `rendement_3jr`, `rendement_5jr`, `rendement_8jr`
#'
#' @examples
#' cohort <- tibble::tibble(
#'   persoonsgebonden_nummer = c("S1", "S2"),
#'   eerstejaar_instelling = 2020L
#' )
#' diploma <- tibble::tibble(
#'   persoonsgebonden_nummer = "S1",
#'   jaar_eerste_diploma = 2022L,
#'   verblijfsjaar_eerste_diploma = 3L,
#'   diploma = "Diploma behaald (excl. propedeuse)"
#' )
#' bereken_rendement(cohort, diploma)
#' @export
bereken_rendement <- function(cohorten_instroom, diploma_behaald) {
  cohorten_instroom |>
    dplyr::left_join(diploma_behaald, by = "persoonsgebonden_nummer") |>
    dplyr::select(
      persoonsgebonden_nummer,
      eerstejaar_instelling,
      jaar_eerste_diploma,
      verblijfsjaar_eerste_diploma,
      diploma
    ) |>
    dplyr::mutate(
      rendement_xjaar = jaar_eerste_diploma - eerstejaar_instelling + 1,

      rendement_3jr = dplyr::case_when(
        is.na(jaar_eerste_diploma) ~ "Geen diploma",
        ## diplomajaar voor instroomjaar kan voorkomen door data-inconsistentie
        jaar_eerste_diploma <
          eerstejaar_instelling ~ "Onbekend (diplomajaar voor instroomjaar)",
        rendement_xjaar <= 3 ~ "Diploma binnen 3 jaar",
        rendement_xjaar > 3 ~ "Diploma na 3 jaar"
      ),

      rendement_5jr = dplyr::case_when(
        is.na(jaar_eerste_diploma) ~ "Geen diploma",
        jaar_eerste_diploma <
          eerstejaar_instelling ~ "Onbekend (diplomajaar voor instroomjaar)",
        rendement_xjaar <= 5 ~ "Diploma binnen 5 jaar",
        rendement_xjaar > 5 ~ "Diploma na 5 jaar"
      ),

      rendement_8jr = dplyr::case_when(
        is.na(jaar_eerste_diploma) ~ "Geen diploma",
        jaar_eerste_diploma <
          eerstejaar_instelling ~ "Onbekend (diplomajaar voor instroomjaar)",
        rendement_xjaar <= 8 ~ "Diploma binnen 8 jaar",
        rendement_xjaar > 8 ~ "Diploma na 8 jaar"
      )
    ) |>
    dplyr::mutate(dplyr::across(dplyr::starts_with("rendement"), as.factor))
}

## --- maak_diploma_behaald ---

test_that("behoudt alleen afgeronde diploma's en niet propedeuse", {
  basisbestand <- tibble(
    persoonsgebonden_nummer = c("A", "B"),
    soort_diploma_instelling = c(
      BACHELOR_DIPLOMA,
      "Propedeusediploma binnen de actuele instelling"
    ),
    diplomajaar = c(2022L, 2020L),
    verblijfsjaar_actuele_instelling = c(3L, 1L)
  )

  result <- maak_diploma_behaald(basisbestand)

  expect_equal(nrow(result), 1)
  expect_equal(result$persoonsgebonden_nummer, "A")
})

test_that("accepteert alle 12 diplomasoorten", {
  alle_soorten <- c(
    "Hoofd-bachelor-diploma binnen de actuele instelling",
    "Neven-bachelor-diploma binnen de actuele instelling",
    "Hoofd-master-diploma binnen de actuele instelling",
    "Neven-master-diploma binnen de actuele instelling",
    "Hoofd-doctoraal-diploma binnen de actuele instelling",
    "Neven-doctoraal-diploma binnen de actuele instelling",
    "Hoofddiploma beroepsfase/voortgezet binnen de actuele instelling",
    "Nevendiploma beroepsfase/voortgezet binnen de actuele instelling",
    "Hoofddiploma associate degree binnen de actuele instelling",
    "Nevendiploma associate degree binnen de actuele instelling",
    "Hoofddiploma postinitiele master binnen de actuele instelling",
    "Nevendiploma postinitiele master binnen de actuele instelling"
  )

  basisbestand <- tibble(
    persoonsgebonden_nummer = as.character(seq_along(alle_soorten)),
    soort_diploma_instelling = alle_soorten,
    diplomajaar = 2022L,
    verblijfsjaar_actuele_instelling = 3L
  )

  result <- maak_diploma_behaald(basisbestand)

  expect_equal(nrow(result), 12)
})

test_that("sluit studenten zonder diploma uit", {
  basisbestand <- tibble(
    persoonsgebonden_nummer = "A",
    soort_diploma_instelling = NA_character_,
    diplomajaar = NA_integer_,
    verblijfsjaar_actuele_instelling = 3L
  )

  result <- maak_diploma_behaald(basisbestand)

  expect_equal(nrow(result), 0)
})

test_that("zet diplomajaar 0 om naar NA", {
  basisbestand <- tibble(
    persoonsgebonden_nummer = "A",
    soort_diploma_instelling = BACHELOR_DIPLOMA,
    diplomajaar = 0L,
    verblijfsjaar_actuele_instelling = 3L
  )

  result <- maak_diploma_behaald(basisbestand)

  expect_true(is.na(result$jaar_eerste_diploma))
})

test_that("kiest het vroegste diploma bij meerdere rijen", {
  basisbestand <- tibble(
    persoonsgebonden_nummer = c("A", "A"),
    soort_diploma_instelling = BACHELOR_DIPLOMA,
    diplomajaar = c(2024L, 2022L),
    verblijfsjaar_actuele_instelling = c(5L, 3L)
  )

  result <- maak_diploma_behaald(basisbestand)

  expect_equal(nrow(result), 1)
  expect_equal(result$jaar_eerste_diploma, 2022)
  expect_equal(result$verblijfsjaar_eerste_diploma, 3L)
})

test_that("bevat de juiste uitvoerkolommen", {
  basisbestand <- tibble(
    persoonsgebonden_nummer = "A",
    soort_diploma_instelling = BACHELOR_DIPLOMA,
    diplomajaar = 2022L,
    verblijfsjaar_actuele_instelling = 3L
  )

  result <- maak_diploma_behaald(basisbestand)

  expect_named(
    result,
    c(
      "persoonsgebonden_nummer",
      "jaar_eerste_diploma",
      "verblijfsjaar_eerste_diploma",
      "diploma"
    )
  )
  expect_equal(result$diploma, "Diploma behaald (excl. propedeuse)")
})


## --- bereken_rendement ---

test_that("berekent rendement_xjaar correct", {
  cohorten_instroom <- tibble(
    persoonsgebonden_nummer = "A",
    eerstejaar_instelling = 2019
  )
  diploma_behaald <- tibble(
    persoonsgebonden_nummer = "A",
    jaar_eerste_diploma = 2022,
    verblijfsjaar_eerste_diploma = 3L,
    diploma = "Diploma behaald (excl. propedeuse)"
  )

  result <- bereken_rendement(cohorten_instroom, diploma_behaald)

  ## 2022 - 2019 + 1 = 4
  expect_equal(as.numeric(as.character(result$rendement_xjaar)), 4)
})

test_that("categoriseert rendement_3jr correct inclusief grenswaarden", {
  cohorten_instroom <- tibble(
    persoonsgebonden_nummer = c("A", "B", "C"),
    eerstejaar_instelling = 2015
  )
  diploma_behaald <- tibble(
    persoonsgebonden_nummer = c("A", "B"),
    ## A: xjaar = 2017 - 2015 + 1 = 3 (exact op grens -> binnen 3 jaar)
    ## B: xjaar = 2018 - 2015 + 1 = 4 (net over grens -> na 3 jaar)
    jaar_eerste_diploma = c(2017, 2018),
    verblijfsjaar_eerste_diploma = c(3L, 4L),
    diploma = "Diploma behaald (excl. propedeuse)"
  )

  result <- bereken_rendement(cohorten_instroom, diploma_behaald)

  expect_equal(
    as.character(result$rendement_3jr[result$persoonsgebonden_nummer == "A"]),
    "Diploma binnen 3 jaar"
  )
  expect_equal(
    as.character(result$rendement_3jr[result$persoonsgebonden_nummer == "B"]),
    "Diploma na 3 jaar"
  )
  expect_equal(
    as.character(result$rendement_3jr[result$persoonsgebonden_nummer == "C"]),
    "Geen diploma"
  )
})

test_that("categoriseert rendement_5jr correct inclusief grenswaarden", {
  cohorten_instroom <- tibble(
    persoonsgebonden_nummer = c("A", "B"),
    eerstejaar_instelling = 2015
  )
  diploma_behaald <- tibble(
    persoonsgebonden_nummer = c("A", "B"),
    ## A: xjaar = 5 (exact op grens)
    ## B: xjaar = 6 (net over grens)
    jaar_eerste_diploma = c(2019, 2020),
    verblijfsjaar_eerste_diploma = c(5L, 6L),
    diploma = "Diploma behaald (excl. propedeuse)"
  )

  result <- bereken_rendement(cohorten_instroom, diploma_behaald)

  expect_equal(
    as.character(result$rendement_5jr[result$persoonsgebonden_nummer == "A"]),
    "Diploma binnen 5 jaar"
  )
  expect_equal(
    as.character(result$rendement_5jr[result$persoonsgebonden_nummer == "B"]),
    "Diploma na 5 jaar"
  )
})

test_that("categoriseert rendement_8jr correct inclusief grenswaarden", {
  cohorten_instroom <- tibble(
    persoonsgebonden_nummer = c("A", "B"),
    eerstejaar_instelling = 2015
  )
  diploma_behaald <- tibble(
    persoonsgebonden_nummer = c("A", "B"),
    jaar_eerste_diploma = c(2022, 2023),
    verblijfsjaar_eerste_diploma = c(8L, 9L),
    diploma = "Diploma behaald (excl. propedeuse)"
  )

  result <- bereken_rendement(cohorten_instroom, diploma_behaald)

  expect_equal(
    as.character(result$rendement_8jr[result$persoonsgebonden_nummer == "A"]),
    "Diploma binnen 8 jaar"
  )
  expect_equal(
    as.character(result$rendement_8jr[result$persoonsgebonden_nummer == "B"]),
    "Diploma na 8 jaar"
  )
})

test_that("geeft 'Geen diploma' voor studenten zonder diploma", {
  cohorten_instroom <- tibble(
    persoonsgebonden_nummer = "A",
    eerstejaar_instelling = 2019
  )
  diploma_behaald <- tibble(
    persoonsgebonden_nummer = character(0),
    jaar_eerste_diploma = integer(0),
    verblijfsjaar_eerste_diploma = integer(0),
    diploma = character(0)
  )

  result <- bereken_rendement(cohorten_instroom, diploma_behaald)

  expect_equal(as.character(result$rendement_3jr), "Geen diploma")
  expect_equal(as.character(result$rendement_5jr), "Geen diploma")
  expect_equal(as.character(result$rendement_8jr), "Geen diploma")
})

test_that("geeft factorkolommen terug voor alle rendement-indicatoren", {
  cohorten_instroom <- tibble(
    persoonsgebonden_nummer = "A",
    eerstejaar_instelling = 2019
  )
  diploma_behaald <- tibble(
    persoonsgebonden_nummer = "A",
    jaar_eerste_diploma = 2022,
    verblijfsjaar_eerste_diploma = 3L,
    diploma = "Diploma behaald (excl. propedeuse)"
  )

  result <- bereken_rendement(cohorten_instroom, diploma_behaald)

  expect_true(is.factor(result$rendement_3jr))
  expect_true(is.factor(result$rendement_5jr))
  expect_true(is.factor(result$rendement_8jr))
})

test_that("markeert als 'Onbekend' als diplomajaar voor instroomjaar ligt", {
  cohorten_instroom <- tibble(
    persoonsgebonden_nummer = "A",
    eerstejaar_instelling = 2020
  )
  diploma_behaald <- tibble(
    persoonsgebonden_nummer = "A",
    jaar_eerste_diploma = 2018,
    verblijfsjaar_eerste_diploma = 1L,
    diploma = "Diploma behaald (excl. propedeuse)"
  )

  result <- bereken_rendement(cohorten_instroom, diploma_behaald)

  expect_true(grepl(
    "Onbekend",
    as.character(result$rendement_3jr)
  ))
})

---
title: "Aan de slag met staat1cho"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Aan de slag met staat1cho}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
library(staat1cho)
```

## Wat doet dit package?

`staat1cho` berekent vier studie-indicatoren per instroomcohort op basis van de 1CHO-aanlevering van DUO:

- **Instroom**: nieuwe studenten per cohortjaar
- **Rendement**: diploma behaald binnen 3, 5 of 8 jaar
- **Uitval**: student uitgeschreven zonder diploma binnen 1 of 3 jaar
- **Studiewissel**: student gewisseld van opleiding binnen 1 of 3 jaar

## De pipeline

De functies werken in vaste volgorde. Elke stap bouwt voort op de vorige.

```
maak_basisbestand()
    └── maak_instroom_cohort()  ──┐
    └── maak_diploma_behaald()  ──┼── bereken_rendement()   ──┐
                                  ├── bereken_uitval()       ──┼── combineer_indicatoren()
                                  └── bereken_studiewissel() ──┘
```

## Voorbeeld met synthetische data

In productie begint de pipeline met `maak_basisbestand()`, die het 1CHO CSV-bestand inleest. Hier bouwen we een minibestand handmatig om de stappen te laten zien.

```{r basisbestand}
library(tibble)

## Drie studenten, twee cohortjaren
## S1: behaalt diploma in 2022 (binnen 3 jaar na instroom 2020)
## S2: valt uit na 1 jaar (verschijnt alleen in 2020)
## S3: wisselt opleiding na 1 jaar (andere opleidingscode in 2021)

hoofd <- "hoofdinschrijving binnen het domein actuele instelling"
bachelor_diploma <- "Hoofd-bachelor-diploma binnen de actuele instelling"

basisbestand <- tibble(
  persoonsgebonden_nummer = c("S1", "S1", "S2", "S3", "S3"),
  inschrijvingsjaar       = c(2020L, 2021L, 2020L, 2020L, 2021L),
  verblijfsjaar_actuele_instelling = c(1L, 2L, 1L, 1L, 2L),
  soort_hoger_onderwijs   = "hbo",
  soort_inschrijving_actuele_instelling = hoofd,
  soort_inschrijving_actuele_instelling_label = hoofd,
  diplomajaar             = c(NA_integer_, 2022L, NA_integer_, NA_integer_, NA_integer_),
  soort_diploma_instelling = c(NA, bachelor_diploma, NA, NA, NA),
  soort_diploma_instelling_label = c(NA, bachelor_diploma, NA, NA, NA),
  geslacht_label = "man",
  opleidingsvorm = "voltijd",
  opleidingsvorm_label = "voltijd",
  indicatie_internationale_student_label = "geen internationale student",
  indicatie_eer_actueel_label = "geen EER-student",
  croho_onderdeel_actuele_opleiding = "techniek",
  croho_onderdeel_actuele_opleiding_label = "techniek",
  opleiding_actueel_equivalent = c("34401", "34401", "34401", "34401", "39999"),
  type_hoger_onderwijs_binnen_soort_hoger_onderwijs = "ba",
  leeftijd_per_peildatum_1_oktober = 19L,
  postcodecijfers_student_op_1_oktober = "4818",
  postcodecijfers_van_de_hoogste_vooropl_voor_het_ho = "4818",
  locatie_label = "Breda"
)
```

### Stap 1: instroomcohort

`maak_instroom_cohort()` filtert op nieuwe eerstejaarsstudenten. Studenten die al eerder waren ingeschreven (verblijfsjaar > 1) vallen af.

```{r instroom}
cohorten <- maak_instroom_cohort(basisbestand, soort_ho = "hbo")
cohorten[, c("persoonsgebonden_nummer", "inschrijvingsjaar", "eerstejaar_instelling")]
```

### Stap 2: diploma bepalen

`maak_diploma_behaald()` zoekt het vroegste diploma per student.

```{r diploma}
diploma <- maak_diploma_behaald(basisbestand)
diploma
```

S1 heeft een diploma in 2022, S2 en S3 niet.

### Stap 3: rendement

`bereken_rendement()` berekent of een student binnen 3, 5 of 8 jaar een diploma heeft behaald.

```{r rendement}
rendement <- bereken_rendement(cohorten, diploma)
rendement[, c("persoonsgebonden_nummer", "rendement_3jr", "rendement_5jr", "rendement_8jr")]
```

### Stap 4: uitval

`bereken_uitval()` bepaalt de status van elke student: diploma behaald, zittend, of uitgevallen.

```{r uitval}
## peiljaar = het jaar na het laatste jaar in de data
uitval <- bereken_uitval(basisbestand, diploma, cohorten, jaar = 2023L)
uitval[, c("persoonsgebonden_nummer", "status", "uitval_1jr", "uitval_3jr")]
```

### Stap 5: studiewissel

`bereken_studiewissel()` detecteert of een student binnen 1 of 3 jaar van opleiding is gewisseld.

```{r wissel}
wissel <- bereken_studiewissel(basisbestand, cohorten, diploma, uitval)
wissel[, c("persoonsgebonden_nummer", "studiewissel_1jr", "studiewissel_3jr")]
```

### Stap 6: combineren

`combineer_indicatoren()` voegt alle stappen samen tot één analysebestand.

```{r combineer, warning = FALSE}
indicatoren <- combineer_indicatoren(cohorten, rendement, uitval, wissel)
indicatoren[, c("inschrijvingsjaar", "opleidingscode", "status", "rendement", "uitval", "studiewissel")]
```

## Dashboard

Voor interactieve verkenning van de data kun je het dashboard starten:

```{r dashboard, eval = FALSE}
start_dashboard()
```

Het dashboard vraagt om een 1CHO CSV-bestand en berekent automatisch alle indicatoren. Vanuit het dashboard kun je filteren op jaar, locatie, sector en opleiding, en de gefilterde data downloaden als CSV.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/instroom.R
\name{maak_instroom_cohort}
\alias{maak_instroom_cohort}
\title{Maak een eerstejaarscohort per instelling}
\usage{
maak_instroom_cohort(basisbestand, soort_ho)
}
\arguments{
\item{basisbestand}{Tibble zoals gemaakt door \code{\link[=maak_basisbestand]{maak_basisbestand()}}}

\item{soort_ho}{Character vector met toegestane waarden van
\code{soort_hoger_onderwijs}, bijv. \code{c("hoger beroepsonderwijs", "hbo")}}
}
\value{
Een tibble met één rij per student, aangevuld met kolom
\code{eerstejaar_instelling} (= inschrijvingsjaar). Gooit een fout als er
dubbele persoonsgebonden nummers zijn.
}
\description{
Filtert het basisbestand op het opgegeven soort hoger onderwijs,
hoofdinschrijvingen en eerste verblijfsjaar.
}
\examples{
basis <- tibble::tibble(
  persoonsgebonden_nummer = c("S1", "S2", "S3"),
  soort_hoger_onderwijs = c("hbo", "wo", "hbo"),
  soort_inschrijving_actuele_instelling_label =
    "hoofdinschrijving binnen het domein actuele instelling",
  verblijfsjaar_actuele_instelling = 1L,
  inschrijvingsjaar = 2020L,
  soort_diploma_instelling_label = NA_character_
)
maak_instroom_cohort(basis, "hbo")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rendement.R
\name{maak_diploma_behaald}
\alias{maak_diploma_behaald}
\title{Maak een bestand met het vroegst behaalde diploma per student}
\usage{
maak_diploma_behaald(basisbestand)
}
\arguments{
\item{basisbestand}{Tibble zoals gemaakt door \code{\link[=maak_basisbestand]{maak_basisbestand()}}}
}
\value{
Een tibble met één rij per student met kolommen
\code{persoonsgebonden_nummer}, \code{jaar_eerste_diploma},
\code{verblijfsjaar_eerste_diploma} en \code{diploma}. Gooit een fout bij
dubbele persoonsgebonden nummers.
}
\description{
Filtert het basisbestand op diplomasoorten die gelden als afgeronde opleiding
en behoudt per student alleen het eerste diploma op basis van diplomajaar.
}
\examples{
basis <- tibble::tibble(
  persoonsgebonden_nummer = c("S1", "S2"),
  soort_diploma_instelling = c(
    "Hoofd-bachelor-diploma binnen de actuele instelling",
    NA_character_
  ),
  diplomajaar = c(2022L, NA_integer_),
  verblijfsjaar_actuele_instelling = c(3L, 1L)
)
maak_diploma_behaald(basis)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rendement.R
\name{bereken_rendement}
\alias{bereken_rendement}
\title{Bereken rendementsindicatoren per cohort}
\usage{
bereken_rendement(cohorten_instroom, diploma_behaald)
}
\arguments{
\item{cohorten_instroom}{Tibble zoals gemaakt door \code{\link[=maak_instroom_cohort]{maak_instroom_cohort()}}}

\item{diploma_behaald}{Tibble zoals gemaakt door \code{\link[=maak_diploma_behaald]{maak_diploma_behaald()}}}
}
\value{
Een tibble met kolommen \code{persoonsgebonden_nummer},
\code{eerstejaar_instelling}, \code{jaar_eerste_diploma},
\code{verblijfsjaar_eerste_diploma}, \code{diploma}, \code{rendement_xjaar},
en factorkolommen \code{rendement_3jr}, \code{rendement_5jr}, \code{rendement_8jr}
}
\description{
Koppelt diplomagegevens aan het instroomcohort en berekent of een student
binnen 3, 5 of 8 jaar een diploma heeft behaald.
}
\examples{
cohort <- tibble::tibble(
  persoonsgebonden_nummer = c("S1", "S2"),
  eerstejaar_instelling = 2020L
)
diploma <- tibble::tibble(
  persoonsgebonden_nummer = "S1",
  jaar_eerste_diploma = 2022L,
  verblijfsjaar_eerste_diploma = 3L,
  diploma = "Diploma behaald (excl. propedeuse)"
)
bereken_rendement(cohort, diploma)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/uitval.R
\name{bereken_uitval}
\alias{bereken_uitval}
\title{Bereken uitvalindicatoren per cohort}
\usage{
bereken_uitval(basisbestand, diploma_behaald, cohorten_instroom, jaar)
}
\arguments{
\item{basisbestand}{Tibble zoals gemaakt door \code{\link[=maak_basisbestand]{maak_basisbestand()}}}

\item{diploma_behaald}{Tibble zoals gemaakt door \code{\link[=maak_diploma_behaald]{maak_diploma_behaald()}}}

\item{cohorten_instroom}{Tibble zoals gemaakt door \code{\link[=maak_instroom_cohort]{maak_instroom_cohort()}}}

\item{jaar}{Integer, peiljaar van de analyse (bijv. \code{2025}). Studenten die
in \code{jaar - 1} nog ingeschreven staan, gelden als zittend.}
}
\value{
Een tibble met kolommen \code{persoonsgebonden_nummer},
\code{laatste_jaar_inschrijving}, \code{diploma}, \code{status} (factor:
Diploma behaald / Zittend / Uitgevallen), \code{uitval_xjr} (jaar van uitval
t.o.v. instroomjaar), \code{uitval_1jr} en \code{uitval_3jr} (factoren).
Gooit een fout bij dubbele studenten of ontbrekende statussen.
}
\description{
Bepaalt voor elke student in het instroomcohort of zij zijn uitgevallen,
zittend of gediplomeerd. Uitval wordt gemarkeerd als een student niet meer
ingeschreven staat en geen diploma heeft.
}
\examples{
basis <- tibble::tibble(
  persoonsgebonden_nummer = c("S1", "S1", "S2"),
  inschrijvingsjaar = c(2020L, 2021L, 2020L),
  soort_inschrijving_actuele_instelling = "hoofdinschrijving"
)
diploma <- tibble::tibble(
  persoonsgebonden_nummer = "S1",
  jaar_eerste_diploma = 2022L,
  verblijfsjaar_eerste_diploma = 3L,
  diploma = "Diploma behaald (excl. propedeuse)"
)
cohort <- tibble::tibble(
  persoonsgebonden_nummer = c("S1", "S2"),
  eerstejaar_instelling = 2020L
)
bereken_uitval(basis, diploma, cohort, jaar = 2023L)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dashboard.R
\name{start_dashboard}
\alias{start_dashboard}
\title{Start het Staat van Onderwijsinstelling dashboard}
\usage{
start_dashboard()
}
\value{
No return value, called for side effects.
}
\description{
Opent de interactieve Shiny app waarmee je een 1CHO CSV-bestand kunt
uploaden en studie-indicatoren kunt verkennen.
}
\examples{
if (interactive()) {
  start_dashboard()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/studiewissel.R
\name{bereken_studiewissel}
\alias{bereken_studiewissel}
\title{Bereken studiewisselindicatoren per cohort}
\usage{
bereken_studiewissel(
  basisbestand,
  cohorten_instroom,
  diploma_behaald,
  uitval_indicatoren
)
}
\arguments{
\item{basisbestand}{Tibble zoals gemaakt door \code{\link[=maak_basisbestand]{maak_basisbestand()}}}

\item{cohorten_instroom}{Tibble zoals gemaakt door \code{\link[=maak_instroom_cohort]{maak_instroom_cohort()}}}

\item{diploma_behaald}{Tibble zoals gemaakt door \code{\link[=maak_diploma_behaald]{maak_diploma_behaald()}}}

\item{uitval_indicatoren}{Tibble zoals gemaakt door \code{\link[=bereken_uitval]{bereken_uitval()}}}
}
\value{
Een tibble met kolommen \code{persoonsgebonden_nummer},
\code{studiewissel_1jr}, \code{studiewissel_3jr} (factoren) en aanvullende
switch-kolommen met de opleiding, opleidingsvorm, niveau en sector na
de wissel
}
\description{
Bepaalt per student of zij binnen 1 of 3 jaar van opleiding zijn gewisseld.
Studenten die al zijn uitgevallen of gediplomeerd in de meetperiode worden
buiten beschouwing gelaten.
}
\examples{
basis <- tibble::tibble(
  persoonsgebonden_nummer = c("S1", "S1"),
  verblijfsjaar_actuele_instelling = c(1L, 2L),
  soort_inschrijving_actuele_instelling_label =
    "hoofdinschrijving binnen het domein actuele instelling",
  opleiding_actueel_equivalent = "34401",
  inschrijvingsjaar = c(2020L, 2021L),
  opleidingsvorm = "voltijd",
  type_hoger_onderwijs_binnen_soort_hoger_onderwijs = "bachelor",
  croho_onderdeel_actuele_opleiding = "techniek"
)
cohort <- tibble::tibble(
  persoonsgebonden_nummer = "S1",
  inschrijvingsjaar = 2020L,
  eerstejaar_instelling = 2020L,
  soort_diploma_instelling_label = NA_character_,
  opleidingsvorm_label = "voltijd"
)
diploma <- tibble::tibble(
  persoonsgebonden_nummer = character(0),
  jaar_eerste_diploma = integer(0),
  verblijfsjaar_eerste_diploma = integer(0),
  diploma = character(0)
)
uitval <- tibble::tibble(
  persoonsgebonden_nummer = "S1",
  uitval_xjr = NA_real_
)
bereken_studiewissel(basis, cohort, diploma, uitval)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/staat1cho-package.R
\docType{package}
\name{staat1cho-package}
\alias{staat1cho}
\alias{staat1cho-package}
\title{staat1cho: Studie-indicatoren op basis van 1CHO-data}
\description{
Berekent instroom-, rendement-, uitval- en studiewisselindicatoren
vanuit de 1CHO-aanlevering van DUO.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/cedanl/staat-van-onderwijsinstelling}
  \item Report bugs at \url{https://github.com/cedanl/staat-van-onderwijsinstelling/issues}
}

}
\author{
\strong{Maintainer}: Aslam Tanjung \email{aslam.tanjung@surf.nl}

Authors:
\itemize{
  \item Veerle van Son
  \item Damiëtte Bakx-van den Brink
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/studiewissel.R
\name{bereken_wissel_xjr}
\alias{bereken_wissel_xjr}
\title{Bereken studiewissel voor een specifiek verblijfsjaar-paar}
\usage{
bereken_wissel_xjr(
  basisbestand,
  zittend,
  verblijfsjaren,
  doeljaar,
  label_gewisseld,
  label_niet,
  suffix
)
}
\arguments{
\item{basisbestand}{Tibble zoals gemaakt door \code{\link[=maak_basisbestand]{maak_basisbestand()}}}

\item{zittend}{Tibble met de populatie waarvoor de wissel bepaald wordt}

\item{verblijfsjaren}{Integer vector van lengte 2: begin- en eindjaar van
het meetvenster, bijv. \code{c(1, 2)} of \code{c(1, 4)}}

\item{doeljaar}{Integer, het verblijfsjaar waarop de wissel wordt gemeten}

\item{label_gewisseld}{Character, label als student gewisseld is}

\item{label_niet}{Character, label als student niet gewisseld is}

\item{suffix}{Character, achtervoegsel voor de kolomnamen (\code{"1jr"} of
\code{"3jr"})}
}
\value{
Een tibble met alleen de studenten die gewisseld zijn, met kolommen
voor wissel, nieuwe opleidingscode, opleidingsvorm, niveau en sector
}
\description{
Hulpfunctie die de switchlogica hergebruikt voor zowel de 1jr- als
3jr-meting. Een wissel wordt vastgesteld als een student in jaar \code{doeljaar}
een andere opleidingscode heeft dan in jaar 1, en het verschil in
kalenderjaren gelijk is aan \code{doeljaar - 1}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/instroom.R
\name{maak_basisbestand}
\alias{maak_basisbestand}
\title{Lees het 1CHO-bestand in en voeg label-kolommen toe}
\usage{
maak_basisbestand(pad_invoer)
}
\arguments{
\item{pad_invoer}{Pad naar het semicolongescheiden CSV-bestand (UTF-8)}
}
\value{
Een tibble met alle 1CHO-regels plus extra \verb{_label}-kolommen die de
originele categorische waarden bewaren voor gebruik in rapportages
}
\description{
Leest een semicolongescheiden CSV-bestand (UTF-8) in en voegt
extra \verb{_label}-kolommen toe voor gebruik in rapportages. Het pakket
bevat een klein synthetisch voorbeeldbestand zonder echte persoonsgegevens
(\code{inst/extdata/voorbeeld_1cho.csv}).
}
\examples{
# voorbeeld_1cho.csv is a small synthetic dataset bundled with the package
pad <- system.file("extdata/voorbeeld_1cho.csv", package = "staat1cho")
basis <- suppressMessages(maak_basisbestand(pad))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/combineer.R
\name{combineer_indicatoren}
\alias{combineer_indicatoren}
\title{Combineer alle indicatoren tot een analysebestand}
\usage{
combineer_indicatoren(
  cohorten_instroom,
  rendement_indicatoren,
  uitval_indicatoren,
  studiewissel_indicatoren
)
}
\arguments{
\item{cohorten_instroom}{Tibble zoals gemaakt door \code{\link[=maak_instroom_cohort]{maak_instroom_cohort()}}}

\item{rendement_indicatoren}{Tibble zoals gemaakt door \code{\link[=bereken_rendement]{bereken_rendement()}}}

\item{uitval_indicatoren}{Tibble zoals gemaakt door \code{\link[=bereken_uitval]{bereken_uitval()}}}

\item{studiewissel_indicatoren}{Tibble zoals gemaakt door
\code{\link[=bereken_studiewissel]{bereken_studiewissel()}}}
}
\value{
Een tibble met een rij per student en gecombineerde indicator-
kolommen, klaar voor rapportage. Bevat o.a. \code{status}, \code{rendement},
\code{uitval}, \code{studiewissel} en alle onderliggende deelscores.
}
\description{
Voegt rendement-, uitval- en studiewisselindicatoren samen met het
instroomcohort. Past kolomnamen en factorniveaus aan voor gebruik in
rapportages.
}
\examples{
cohort <- tibble::tibble(
  persoonsgebonden_nummer = "S1",
  inschrijvingsjaar = 2020L,
  eerstejaar_instelling = 2020L,
  geslacht_label = "man",
  locatie_label = "Breda",
  opleiding_actueel_equivalent = "34401",
  opleidingsvorm_label = "voltijd",
  type_hoger_onderwijs_binnen_soort_hoger_onderwijs = "ba",
  indicatie_internationale_student_label = "geen internationale student",
  indicatie_eer_actueel_label = "geen EER-student",
  croho_onderdeel_actuele_opleiding_label = "techniek",
  leeftijd_per_peildatum_1_oktober = 19L,
  postcodecijfers_student_op_1_oktober = "4818",
  postcodecijfers_van_de_hoogste_vooropl_voor_het_ho = "4818",
  soort_diploma_instelling_label = NA_character_
)
rendement <- tibble::tibble(
  persoonsgebonden_nummer = "S1",
  eerstejaar_instelling = 2020L,
  jaar_eerste_diploma = NA_real_,
  verblijfsjaar_eerste_diploma = NA_integer_,
  diploma = NA_character_,
  rendement_xjaar = factor(NA_character_),
  rendement_3jr = factor("Geen diploma"),
  rendement_5jr = factor("Geen diploma"),
  rendement_8jr = factor("Geen diploma")
)
uitval <- tibble::tibble(
  persoonsgebonden_nummer = "S1",
  laatste_jaar_inschrijving = NA_real_,
  diploma = NA_character_,
  status = factor("Zittend"),
  uitval_xjr = NA_real_,
  uitval_1jr = factor("Na 1 jaar nog ingeschreven of diploma behaald"),
  uitval_3jr = factor("Na 3 jaar nog ingeschreven of diploma behaald")
)
wissel <- tibble::tibble(
  persoonsgebonden_nummer = "S1",
  studiewissel_1jr = factor("Niet gewisseld binnen 1 jaar"),
  studiewissel_3jr = factor("Niet gewisseld binnen 3 jaar"),
  opleidingscode_na_switch1jr = factor(NA_character_),
  opleidingsvorm_na_switch1jr = factor(NA_character_),
  opleidingsniveau_na_switch1jr = factor(NA_character_),
  sector_na_switch1jr = factor(NA_character_),
  opleidingscode_na_switch3jr = factor(NA_character_),
  opleidingsvorm_na_switch3jr = factor(NA_character_),
  opleidingsniveau_na_switch3jr = factor(NA_character_),
  sector_na_switch3jr = factor(NA_character_)
)
suppressWarnings(combineer_indicatoren(cohort, rendement, uitval, wissel))
}
